<?php
/**
Plugin Name: FraudLabs Pro SMS Verification
Description: SMS verification help merchants to authenticate the client's identity by sending them a SMS for verification.
Version: 1.9.5
Author: FraudLabs Pro
Author URI: https://www.fraudlabspro.com
Text Domain: fraudlabs-pro-sms-verification
*/

defined('DS') or define('DS', DIRECTORY_SEPARATOR);
define('FRAUDLABS_PRO_SMS_VERIFICATION_ROOT', __DIR__ . DS);

if (!class_exists('FraudLabsPro_SMS_Verification')) :

class FraudLabsPro_SMS_Verification {

	private $countries = ['AF' => 'Afghanistan', 'AX' => 'Aland Islands', 'AL' => 'Albania', 'DZ' => 'Algeria', 'AS' => 'American Samoa', 'AD' => 'Andorra', 'AO' => 'Angola', 'AI' => 'Anguilla', 'AQ' => 'Antarctica', 'AG' => 'Antigua and Barbuda', 'AR' => 'Argentina', 'AM' => 'Armenia', 'AW' => 'Aruba', 'AU' => 'Australia', 'AT' => 'Austria', 'AZ' => 'Azerbaijan', 'BS' => 'Bahamas', 'BH' => 'Bahrain', 'BD' => 'Bangladesh', 'BB' => 'Barbados', 'BY' => 'Belarus', 'BE' => 'Belgium', 'BZ' => 'Belize', 'BJ' => 'Benin', 'BM' => 'Bermuda', 'BT' => 'Bhutan', 'BO' => 'Bolivia, Plurinational State of', 'BQ' => 'Bonaire, Sint Eustatius and Saba', 'BA' => 'Bosnia and Herzegovina', 'BW' => 'Botswana', 'BV' => 'Bouvet Island', 'BR' => 'Brazil', 'IO' => 'British Indian Ocean Territory', 'BN' => 'Brunei Darussalam', 'BG' => 'Bulgaria', 'BF' => 'Burkina Faso', 'BI' => 'Burundi', 'CV' => 'Cabo Verde', 'KH' => 'Cambodia', 'CM' => 'Cameroon', 'CA' => 'Canada', 'KY' => 'Cayman Islands', 'CF' => 'Central African Republic', 'TD' => 'Chad', 'CL' => 'Chile', 'CN' => 'China', 'CX' => 'Christmas Island', 'CC' => 'Cocos (Keeling) Islands', 'CO' => 'Colombia', 'KM' => 'Comoros', 'CG' => 'Congo', 'CD' => 'Congo, The Democratic Republic of The', 'CK' => 'Cook Islands', 'CR' => 'Costa Rica', 'CI' => 'Cote D\'ivoire', 'HR' => 'Croatia', 'CU' => 'Cuba', 'CW' => 'Curacao', 'CY' => 'Cyprus', 'CZ' => 'Czech Republic', 'DK' => 'Denmark', 'DJ' => 'Djibouti', 'DM' => 'Dominica', 'DO' => 'Dominican Republic', 'EC' => 'Ecuador', 'EG' => 'Egypt', 'SV' => 'El Salvador', 'GQ' => 'Equatorial Guinea', 'ER' => 'Eritrea', 'EE' => 'Estonia', 'ET' => 'Ethiopia', 'FK' => 'Falkland Islands (Malvinas)', 'FO' => 'Faroe Islands', 'FJ' => 'Fiji', 'FI' => 'Finland', 'FR' => 'France', 'GF' => 'French Guiana', 'PF' => 'French Polynesia', 'TF' => 'French Southern Territories', 'GA' => 'Gabon', 'GM' => 'Gambia', 'GE' => 'Georgia', 'DE' => 'Germany', 'GH' => 'Ghana', 'GI' => 'Gibraltar', 'GR' => 'Greece', 'GL' => 'Greenland', 'GD' => 'Grenada', 'GP' => 'Guadeloupe', 'GU' => 'Guam', 'GT' => 'Guatemala', 'GG' => 'Guernsey', 'GN' => 'Guinea', 'GW' => 'Guinea-Bissau', 'GY' => 'Guyana', 'HT' => 'Haiti', 'HM' => 'Heard Island and Mcdonald Islands', 'VA' => 'Holy See', 'HN' => 'Honduras', 'HK' => 'Hong Kong', 'HU' => 'Hungary', 'IS' => 'Iceland', 'IN' => 'India', 'ID' => 'Indonesia', 'IR' => 'Iran, Islamic Republic of', 'IQ' => 'Iraq', 'IE' => 'Ireland', 'IM' => 'Isle of Man', 'IL' => 'Israel', 'IT' => 'Italy', 'JM' => 'Jamaica', 'JP' => 'Japan', 'JE' => 'Jersey', 'JO' => 'Jordan', 'KZ' => 'Kazakhstan', 'KE' => 'Kenya', 'KI' => 'Kiribati', 'KP' => 'Korea, Democratic People\'s Republic of', 'KR' => 'Korea, Republic of', 'KW' => 'Kuwait', 'KG' => 'Kyrgyzstan', 'LA' => 'Lao People\'s Democratic Republic', 'LV' => 'Latvia', 'LB' => 'Lebanon', 'LS' => 'Lesotho', 'LR' => 'Liberia', 'LY' => 'Libya', 'LI' => 'Liechtenstein', 'LT' => 'Lithuania', 'LU' => 'Luxembourg', 'MO' => 'Macao', 'MK' => 'Macedonia, The Former Yugoslav Republic of', 'MG' => 'Madagascar', 'MW' => 'Malawi', 'MY' => 'Malaysia', 'MV' => 'Maldives', 'ML' => 'Mali', 'MT' => 'Malta', 'MH' => 'Marshall Islands', 'MQ' => 'Martinique', 'MR' => 'Mauritania', 'MU' => 'Mauritius', 'YT' => 'Mayotte', 'MX' => 'Mexico', 'FM' => 'Micronesia, Federated States of', 'MD' => 'Moldova, Republic of', 'MC' => 'Monaco', 'MN' => 'Mongolia', 'ME' => 'Montenegro', 'MS' => 'Montserrat', 'MA' => 'Morocco', 'MZ' => 'Mozambique', 'MM' => 'Myanmar', 'NA' => 'Namibia', 'NR' => 'Nauru', 'NP' => 'Nepal', 'NL' => 'Netherlands', 'NC' => 'New Caledonia', 'NZ' => 'New Zealand', 'NI' => 'Nicaragua', 'NE' => 'Niger', 'NG' => 'Nigeria', 'NU' => 'Niue', 'NF' => 'Norfolk Island', 'MP' => 'Northern Mariana Islands', 'NO' => 'Norway', 'OM' => 'Oman', 'PK' => 'Pakistan', 'PW' => 'Palau', 'PS' => 'Palestine, State of', 'PA' => 'Panama', 'PG' => 'Papua New Guinea', 'PY' => 'Paraguay', 'PE' => 'Peru', 'PH' => 'Philippines', 'PN' => 'Pitcairn', 'PL' => 'Poland', 'PT' => 'Portugal', 'PR' => 'Puerto Rico', 'QA' => 'Qatar', 'RE' => 'Reunion', 'RO' => 'Romania', 'RU' => 'Russian Federation', 'RW' => 'Rwanda', 'BL' => 'Saint Barthelemy', 'SH' => 'Saint Helena, Ascension and Tristan Da Cunha', 'KN' => 'Saint Kitts and Nevis', 'LC' => 'Saint Lucia', 'MF' => 'Saint Martin (French Part)', 'PM' => 'Saint Pierre and Miquelon', 'VC' => 'Saint Vincent and The Grenadines', 'WS' => 'Samoa', 'SM' => 'San Marino', 'ST' => 'Sao Tome and Principe', 'SA' => 'Saudi Arabia', 'SN' => 'Senegal', 'RS' => 'Serbia', 'SC' => 'Seychelles', 'SL' => 'Sierra Leone', 'SG' => 'Singapore', 'SX' => 'Sint Maarten (Dutch Part)', 'SK' => 'Slovakia', 'SI' => 'Slovenia', 'SB' => 'Solomon Islands', 'SO' => 'Somalia', 'ZA' => 'South Africa', 'GS' => 'South Georgia and The South Sandwich Islands', 'SS' => 'South Sudan', 'ES' => 'Spain', 'LK' => 'Sri Lanka', 'SD' => 'Sudan', 'SR' => 'Suriname', 'SJ' => 'Svalbard and Jan Mayen', 'SZ' => 'Eswatini', 'SE' => 'Sweden', 'CH' => 'Switzerland', 'SY' => 'Syrian Arab Republic', 'TW' => 'Taiwan, Province of China', 'TJ' => 'Tajikistan', 'TZ' => 'Tanzania, United Republic of', 'TH' => 'Thailand', 'TL' => 'Timor-Leste', 'TG' => 'Togo', 'TK' => 'Tokelau', 'TO' => 'Tonga', 'TT' => 'Trinidad and Tobago', 'TN' => 'Tunisia', 'TR' => 'Turkey', 'TM' => 'Turkmenistan', 'TC' => 'Turks and Caicos Islands', 'TV' => 'Tuvalu', 'UG' => 'Uganda', 'UA' => 'Ukraine', 'AE' => 'United Arab Emirates', 'GB' => 'United Kingdom', 'US' => 'United States', 'UM' => 'United States Minor Outlying Islands', 'UY' => 'Uruguay', 'UZ' => 'Uzbekistan', 'VU' => 'Vanuatu', 'VE' => 'Venezuela, Bolivarian Republic of', 'VN' => 'Viet Nam', 'VG' => 'Virgin Islands, British', 'VI' => 'Virgin Islands, U.S.', 'WF' => 'Wallis and Futuna', 'EH' => 'Western Sahara', 'YE' => 'Yemen', 'ZM' => 'Zambia', 'ZW' => 'Zimbabwe'];

	/**
	* Construct the plugin.
	*/
	public function __construct() {

		// Define user set variables.
		$this->api_key				= get_option('fraudlabs_pro_sms_verification_api_key');
		$this->sms_retries			= get_option('fraudlabs_pro_sms_verification_sms_retries');
		$this->sms_instruction		= get_option('fraudlabs_pro_sms_verification_sms_instruction');
		$this->sms_template			= get_option('fraudlabs_pro_sms_verification_sms_template');
		$this->otp_timeout			= get_option('fraudlabs_pro_sms_verification_otp_timeout');

		add_action('plugins_loaded', array($this, 'init'));
		add_action('admin_menu', array($this, 'admin_page'));
		add_action('register_form', array($this, 'wp_default_register_form'));
		add_action('login_form', array($this, 'wp_default_login_form'));
		add_action('wp_ajax_wp_sms_action_send', array($this, 'wp_sms_action_send'));
		add_action('wp_ajax_wp_sms_action_verify', array($this, 'wp_sms_action_verify'));
		add_action('wp_loaded', array($this, 'callback_external'));
		add_action('admin_enqueue_scripts', array($this, 'plugin_enqueues'));
		add_action('wp_ajax_fraudlabspro_sms_verification_submit_feedback', array($this, 'submit_feedback'));
		add_action('admin_footer_text', array($this, 'admin_footer_text'));

		add_shortcode('flp_sms_verification', array($this, 'flp_sms_form'));
		// Shortcode for sms verification via email.
		add_shortcode('flp-sms-verification-edd', array($this, 'flp_sms_form_edd'));

		// Checks if Contact Form is installed.
		if (class_exists('WPCF7')) {
			wpcf7_add_form_tag('flp_sms_verification', array($this, 'flp_sms_form'));
		}

		// Check if Easy Digital Downloads is installed.
		if (class_exists('EDD_License')) {
			add_action('edd_complete_purchase',  array($this, 'edd_sms_verification_email'));
		}

		if (isset($_REQUEST['action']) && $_REQUEST['action']=='wp_sms_action_send') {
			add_action('wp_ajax_wp_sms_action_send', array($this, 'wp_sms_action_send'));
			do_action('wp_ajax_' . $_REQUEST['action']);
		}
		if (isset($_REQUEST['action']) && $_REQUEST['action']=='wp_sms_action_verify') {
			add_action('wp_ajax_wp_sms_action_verify', array($this, 'wp_sms_action_verify'));
			do_action('wp_ajax_' . $_REQUEST['action']);
		}
		if (isset($_REQUEST['action']) && $_REQUEST['action']=='fraudlabspro_sms_verification_submit_feedback') {
			add_action('wp_ajax_fraudlabspro_sms_verification_submit_feedback', array($this, 'submit_feedback'));
		}
	}

	/**
	* Initialize the plugin.
	*/
	public function init() {
		// Checks if WooCommerce is installed.
		if (class_exists('WC_Integration')) {
			// Include our integration class.
			include_once 'includes/class-wc-fraudlabspro-sms-verification.php';

			// Register the integration.
			add_filter('woocommerce_integrations', array($this, 'add_integration'));
		}
	}

	/**
	 * Add a new integration to WooCommerce.
	 */
	public function add_integration($integrations) {
		$integrations[] = 'WC_Integration_FraudLabs_Pro_Sms_Verification';

		return $integrations;
	}

	/**
	 * Add admin page.
	 */
	function admin_page() {
		add_menu_page('SMS Verification', 'SMS Verification', 'manage_options', 'fraudlabspro-sms-verification', array(&$this, 'admin_options'), 'dashicons-admin-fraudlabspro-sms-verification', 30);
	}

	/**
	 * Content of admin page.
	 */
	function admin_options() {
		if (!is_admin()) {
			$this->write_debug_log('Not logged in as administrator. Settings page will not be shown.');
			return;
		}

		add_action('wp_enqueue_script', 'load_jquery');
		wp_enqueue_script('fraudlabs_pro_sms_verification_chosen_js', 'https://cdnjs.cloudflare.com/ajax/libs/chosen/1.7.0/chosen.jquery.min.js', [], null, true);
		wp_enqueue_style('fraudlabs_pro_sms_verification_chosen_css', esc_url_raw('https://cdnjs.cloudflare.com/ajax/libs/chosen/1.7.0/chosen.min.css'), [], null);
		wp_enqueue_script('fraudlabs_pro_sms_verification_admin_script', plugins_url('/assets/js/script.js', __FILE__), ['jquery'], null, true);

		$tab = (isset($_GET['tab'])) ? sanitize_text_field($_GET['tab']) : 'settings';
		$status = '';

		switch ($tab) {
			// WordPress tab.
			case 'wordpress':

				$wp_default_registration_form = (isset($_POST['wp_default_registration_form'])) ? 1 : 0;
				$wp_default_login_form = (isset($_POST['wp_default_login_form'])) ? 1 : 0;

				if (isset($_POST['form_wp_submitted'])) {
					update_option('fraudlabs_pro_sms_verification_wp_default_registration_form', $wp_default_registration_form);
					update_option('fraudlabs_pro_sms_verification_wp_default_login_form', $wp_default_login_form);

					$status .= '
					<div id="message" class="updated">
						<p>The changes have been successfully saved.</p>
					</div>';
				}

				echo '
				<div class="wrap">
					<h1>FraudLabs Pro SMS Verification</h1>
					<p>
						This plugin helps you to verify the user\'s mobile phone number by using the SMS verification technology. It sends an OTP (One Time Passcode) to the user\'s given phone number, and then this OTP shall be entered into the form to confirm the phone ownership.
					</p>

					' . $this->admin_tabs() . '

					' . $status . '
					<div style="margin-left:20px;">
						<p>You can enable the SMS verification on the input form. Supported forms are listed below:</p>

						<form method="post">
							<table class="form-table">
								<tr>
									<th scope="row">
										<label for="wp_default_registration_form">Enable SMS Verification On WordPress Default Registration Form</label>
									</th>
									<td>
										<input name="wp_default_registration_form" type="checkbox" id="wp_default_registration_form"' . ((get_option('fraudlabs_pro_sms_verification_wp_default_registration_form')) ? ' checked = "checked"' : '') . ' />
									</td>
								</tr>
								<tr>
									<th scope="row">
										<label for="wp_default_login_form">Enable SMS Verification On WordPress Default Login Form</label>
									</th>
									<td>
										<input name="wp_default_login_form" type="checkbox" id="wp_default_login_form"' . ((get_option('fraudlabs_pro_sms_verification_wp_default_login_form')) ? ' checked = "checked"' : '') . ' />
									</td>
								</tr>
							</table>

							<input type="hidden" name="form_wp_submitted" value="" />
							<input type="submit" name="submit" id="submit" class="button button-primary" value="Save Changes" />
						</form>
					</div>
				</div>';
				break;

			// WooCommerce tab.
			case 'woocommerce':
				// Check if WooCommerce is installed.
				if (!class_exists('WC_Integration')) {
					echo "<script>
						jQuery(document).ready(function($) {
							refresh_wc_main_settings();
							jQuery('.chosen').chosen();
							function refresh_wc_main_settings() {
								jQuery('#wc_default_registration_form').prop('disabled', true);
								jQuery('#wc_default_checkout_form').prop('disabled', true);
								jQuery('#wc_sms_trigger').prop('disabled', true);
								jQuery('#wc_trigger_countries_option').prop('disabled', true);
								jQuery('#wc_trigger_countries').prop('disabled', true);
								jQuery('#wc_email_verify').prop('disabled', true);
								jQuery('#wc_email_subject').prop('disabled', true);
								jQuery('#wc_email_content').prop('disabled', true);
								jQuery('#wc_change_order_when_verified').prop('disabled', true);
								jQuery('#wc_verified_status').prop('disabled', true);

								$('.chosen').trigger('chosen:updated');
							}
						});
					</script>";
					$wc_order_statuses[''] = 'No Status Change';
				} else {
					$wc_order_statuses = wc_get_order_statuses();
					$wc_order_statuses[''] = 'No Status Change';
				}

				$wc_default_registration_form = (isset($_POST['wc_default_registration_form'])) ? 1 : 0;
				$wc_default_checkout_form = (isset($_POST['wc_default_checkout_form'])) ? 1 : 0;
				$wc_sms_trigger = (isset($_POST['wc_sms_trigger'])) ? sanitize_text_field($_POST['wc_sms_trigger']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') : 'always');
				$wc_trigger_countries_option = (isset($_POST['wc_trigger_countries_option'])) ? sanitize_text_field($_POST['wc_trigger_countries_option']) : get_option('fraudlabs_pro_sms_verification_wc_trigger_countries_option');
				$wc_trigger_countries = (isset($_POST['wc_trigger_countries'])) ? sanitize_text_field($_POST['wc_trigger_countries']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_wc_trigger_countries') : '');
				$wc_trigger_countries = (!is_array($wc_trigger_countries)) ? [$wc_trigger_countries] : $wc_trigger_countries;
				$wc_email_verify = (isset($_POST['wc_email_verify'])) ? 1 : 0;
				$wc_email_subject = (isset($_POST['wc_email_subject'])) ? sanitize_text_field($_POST['wc_email_subject']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_wc_email_subject') : '');
				$wc_email_content = (isset($_POST['wc_email_content'])) ? $_POST['wc_email_content'] : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_wc_email_content') : '');
				$wc_change_order_when_verified = (isset($_POST['wc_change_order_when_verified'])) ? 1 : 0;
				$wc_verified_status = (isset($_POST['wc_verified_status'])) ? sanitize_text_field($_POST['wc_verified_status']) : (((null !== get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_complete_order_when_verified')) && (get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_complete_order_when_verified') == 'yes')) ? 'wc-completed' : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_wc_verified_status') : ''));

				if (isset($_POST['form_wc_submitted'])) {
					if (($wc_email_content !== '') && (strpos($wc_email_content, 'script') !== false)) {
						$status .= '
						<div id="message" class="error">
							<p>Error in saving the Email Body of WooCommerce tab.</p>
						</div>';
					} elseif (($wc_email_content !== '') && (strpos($wc_email_content, '{email_verification_link}') == false)) {
						$status .= '
						<div id="message" class="error">
							<p>The <strong>{email_verification_link}</strong> tag must be included in the Email Body of WooCommerce tab.</p>
						</div>';
					} else {
						update_option('fraudlabs_pro_sms_verification_wc_default_registration_form', $wc_default_registration_form);
						update_option('fraudlabs_pro_sms_verification_wc_default_checkout_form', $wc_default_checkout_form);
						update_option('fraudlabs_pro_sms_verification_wc_sms_trigger', $wc_sms_trigger);
						update_option('fraudlabs_pro_sms_verification_wc_trigger_countries_option', $wc_trigger_countries_option);
						update_option('fraudlabs_pro_sms_verification_wc_trigger_countries', $wc_trigger_countries);
						update_option('fraudlabs_pro_sms_verification_wc_email_verify', $wc_email_verify);
						update_option('fraudlabs_pro_sms_verification_wc_email_subject', $wc_email_subject);
						update_option('fraudlabs_pro_sms_verification_wc_email_content', $wc_email_content);
						update_option('fraudlabs_pro_sms_verification_wc_change_order_when_verified', $wc_change_order_when_verified);
						update_option('fraudlabs_pro_sms_verification_wc_verified_status', $wc_verified_status);

						if ((isset($_POST['wc_email_verify'])) && (null !== get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_enabled_wc_email_verify'))) {
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_api_key');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_enabled_wc_checkout_form');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_enabled_wc_registration_form');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_sms_instruction');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_sms_retries');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_sms_template');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_sms_trigger');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_sms_trigger_countries');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_enabled_wc_email_verify');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_email_subject');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_email_content');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_enabled_fraudlabs_pro_debug_log');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_otp_timeout');
							delete_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_complete_order_when_verified');

							wp_cache_flush();
						}

						$status .= '
						<div id="message" class="updated">
							<p>The changes have been successfully saved.</p>
						</div>';
					}
				}

				echo '
				<div class="wrap">
					<h1>FraudLabs Pro SMS Verification</h1>
					<p>
						This plugin helps you to verify the user\'s mobile phone number by using the SMS verification technology. It sends an OTP (One Time Passcode) to the user\'s given phone number, and then this OTP shall be entered into the form to confirm the phone ownership.
					</p>

					' . $this->admin_tabs() . '

					' . $status . '
					<div style="margin-left:20px;">
						<p>Please note that you have to install <a href="https://wordpress.org/plugins/woocommerce/" target="_blank">WooCommerce plugin</a> before enabling this feature.</p>

						<div style="border-bottom:1px solid #ccc;">
							<h3>Enable SMS Verification On WooCommerce Registration Form</h3>
						</div>

						<p>Trigger SMS verification on WooCommerce registration page.</p>

						<form method="post">
							<table class="form-table">
								<tr>
									<th scope="row" style="padding-left:35px;">
										<label for="wc_default_registration_form">Enable</label>
									</th>
									<td>
										<input name="wc_default_registration_form" type="checkbox" id="wc_default_registration_form"' . ((get_option('fraudlabs_pro_sms_verification_wc_default_registration_form')) ? ' checked = "checked"' : '') . ' />
									</td>
								</tr>
							</table>

						<p>&nbsp;</p>

						<div style="border-bottom:1px solid #ccc;">
							<h3>Enable SMS Verification On WooCommerce Checkout Form</h3>
						</div>

						<p>Trigger SMS verification on WooCommerce checkout page.</p>
							<table class="form-table">
								<tr>
									<th scope="row" style="padding-left:35px;">
										<label for="wc_default_checkout_form">Enable</label>
									</th>
									<td>
										<input name="wc_default_checkout_form" type="checkbox" id="wc_default_checkout_form"' . ((get_option('fraudlabs_pro_sms_verification_wc_default_checkout_form')) ? ' checked = "checked"' : '') . ' />
									</td>
								</tr>

								<tr id="wc_sms_trigger_section">
									<th scope="row" style="padding-left:35px;"><label for="wc_sms_trigger">Trigger SMS Verification If</label></th>
									<td>
										<select name="wc_sms_trigger" id="wc_sms_trigger" class="regular-text">
											<option value="always"' . ((get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') == 'always') ? ' selected' : '') . '>Always</option>
											<option value="new_client_only"' . ((get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') == 'new_client_only') ? ' selected' : '') . '>New Customer</option>
											<option value="free_email"' . ((get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') == 'free_email') ? ' selected' : '') . '>Free Email</option>
											<option value="new_device"' . ((get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') == 'new_device') ? ' selected' : '') . '>New Device</option>
											<option value="specific_billing_countries"' . ((get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') == 'specific_billing_countries') ? ' selected' : '') . '>Billing Countries</option>
											<option value="specific_shipping_countries"' . ((get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') == 'specific_shipping_countries') ? ' selected' : '') . '>Shipping Countries</option>
											<option value="specific_idd_countries"' . ((get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') == 'specific_idd_countries') ? ' selected' : '') . '>IDD Countries</option>
										</select>
										<p class="description">Field to be checked for SMS verification.</p>
									</td>
								</tr>
								<tr>
									<th scope="row" style="padding-left:35px;"><label>Matches The Following Countries</label></th>
									<td>
										<label>
											<input type="radio" name="wc_trigger_countries_option" id="wc_trigger_countries_option_1" value="1"' . (($wc_trigger_countries_option == 1) ? ' checked' : '') . ' class="input-field">
											All countries
										</label>
										<br /><br />
										<label>
											<input type="radio" name="wc_trigger_countries_option" id="wc_trigger_countries_option_2" value="2"' . (($wc_trigger_countries_option == 2) ? ' checked' : '') . ' class="input-field">
											<select name="wc_trigger_countries[]" id="wc_trigger_countries" data-placeholder="Choose countries..." multiple="true" class="chosen input-field">';
											foreach ($this->countries as $country_code => $country_name) {
													echo '
														<option value="' . $country_code . '"' . (($this->is_in_array($country_code, $wc_trigger_countries)) ? ' selected' : '') . '> ' . $country_name . '</option>';
												}
										echo'
											</select>
										</label>
										<p class="description">Trigger the SMS verification if the above field matches the selected countries.</p>
									</td>
								</tr>
							</table>

						<p>&nbsp;</p>

						<div style="border-bottom:1px solid #ccc;">
							<h3>Enable SMS Verification When An Order Is Flagged For Review</h3>
						</div>

						<p>Trigger SMS verification when an order was flagged for <strong>REVIEW</strong> by FraudLabs Pro which will happen after the checkout process. The system will email the user providing them with the link to complete the SMS verification process.<br /><br /><i>Please note that you have to install <a href="https://wordpress.org/plugins/fraudlabs-pro-for-woocommerce/" target="_blank">FraudLabs Pro for WooCommerce plugin</a> before enabling this feature.</i></p>

							<table class="form-table">
								<tr>
									<th scope="row" style="padding-left:35px;">
										<label for="wc_email_verify">Enable</label>
									</th>
									<td>
										<input name="wc_email_verify" type="checkbox" id="wc_email_verify"' . ((get_option('fraudlabs_pro_sms_verification_wc_email_verify')) ? ' checked = "checked"' : (((null !== get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_enabled_wc_email_verify')) && (get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_enabled_wc_email_verify') == 'yes')) ? ' checked = "checked"' : '')) . ' />
									</td>
								</tr>
								<tr>
									<th scope="row" style="padding-left:35px;"><label for="wc_email_subject">Email Subject</label></th>
									<td>
										<input name="wc_email_subject" type="text" id="wc_email_subject" value="' . ((get_option('fraudlabs_pro_sms_verification_wc_email_subject') != '') ? get_option('fraudlabs_pro_sms_verification_wc_email_subject') : ((null !== get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_email_subject')) ? get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_email_subject') : '')) . '" class="regular-text" />
										<p class="description">Subject or title of the mail.</p>
									</td>
								</tr>
								<tr>
									<th scope="row" style="padding-left:35px;"><label for="wc_email_content">Email Body</label></th>
									<td>
										<textarea id="wc_email_content" name="wc_email_content" rows="4" cols="50">' . ((get_option('fraudlabs_pro_sms_verification_wc_email_content') != '') ? get_option('fraudlabs_pro_sms_verification_wc_email_content') : ((null !== get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_email_content')) ? get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_email_content') : '')) . '</textarea>
										<p class="description">Email body. You must include the {email_verification_link} tag which will be automatically replaced with the link for the SMS verification process.</p>
									</td>
								</tr>
								<tr>
									<th scope="row" style="padding-left:35px;">
										<label for="wc_change_order_when_verified">Enable Order Status Change Upon Verified</label>
									</th>
									<td>
										<input name="wc_change_order_when_verified" type="checkbox" id="wc_change_order_when_verified"' . ((get_option('fraudlabs_pro_sms_verification_wc_change_order_when_verified')) ? ' checked = "checked"' : (((null !== get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_complete_order_when_verified')) && (get_option('wc_settings_woocommerce-fraudlabs-pro-sms-verification_complete_order_when_verified') == 'yes')) ? ' checked = "checked"' : '')) . ' />
									</td>
								</tr>
								<tr>
									<th scope="row" style="padding-left:35px;">
										<label for="wc_verified_status">Change Order Status To</label>
									</th>
									<td>
										<select name="wc_verified_status" id="wc_verified_status">';
										foreach ($wc_order_statuses as $key => $status) {
											echo '
											<option value="' . $key . '"' . (($wc_verified_status == $key) ? ' selected' : '') . '> ' . $status . '</option>';
										}

										echo '
										</select>
										<p class="description">
											Change to this order status if the OTP is successfully verified.
										</p>
									</td>
								</tr>
							</table>

							<input type="hidden" name="form_wc_submitted" value="" />
							<input type="submit" name="submit" id="submit" class="button button-primary" value="Save Changes" />
						</form>
					</div>
				</div>';
				break;

			// Easy Digital Downloads tab.
			case 'edd':
				// Check if Easy Digital Downloads is installed.
				if (!class_exists('EDD_License')) {
					echo "<script>
						jQuery(document).ready(function($) {
							refresh_edd_main_settings();
							function refresh_edd_main_settings() {
								jQuery('#edd_email_verify').prop('disabled', true);
								jQuery('#edd_email_subject').prop('disabled', true);
								jQuery('#edd_email_content').prop('disabled', true);
							}
						});
					</script>";
				}

				$edd_email_verify = (isset($_POST['edd_email_verify'])) ? 1 : 0;
				$edd_email_subject = (isset($_POST['edd_email_subject'])) ? sanitize_text_field($_POST['edd_email_subject']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_edd_email_subject') : '');
				$edd_email_content = (isset($_POST['edd_email_content'])) ? $_POST['edd_email_content'] : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_edd_email_content') : '');

				if (isset($_POST['form_edd_submitted'])) {
					if (($edd_email_content !== '') && (strpos($edd_email_content, 'script') !== false)) {
						$status .= '
						<div id="message" class="error">
							<p>Error in saving the Email Body of Easy Digital Downloads tab.</p>
						</div>';
					} elseif (($edd_email_content !== '') && (strpos($edd_email_content, '{email_verification_link}') == false)) {
						$status .= '
						<div id="message" class="error">
							<p>The <strong>{email_verification_link}</strong> tag must be included in the Email Body of Easy Digital Downloads tab.</p>
						</div>';
					} else {
						update_option('fraudlabs_pro_sms_verification_edd_email_verify', $edd_email_verify);
						update_option('fraudlabs_pro_sms_verification_edd_email_subject', $edd_email_subject);
						update_option('fraudlabs_pro_sms_verification_edd_email_content', $edd_email_content);

						$status .= '
						<div id="message" class="updated">
							<p>The changes have been successfully saved.</p>
						</div>';
					}
				}

				echo '
				<div class="wrap">
					<h1>FraudLabs Pro SMS Verification</h1>
					<p>
						This plugin helps you to verify the user\'s mobile phone number by using the SMS verification technology. It sends an OTP (One Time Passcode) to the user\'s given phone number, and then this OTP shall be entered into the form to confirm the phone ownership.
					</p>

					' . $this->admin_tabs() . '

					' . $status . '
					<div style="margin-left:20px;">
						<p>Please note that you have to install <a href="https://wordpress.org/plugins/easy-digital-downloads/" target="_blank">Easy Digital Downloads plugin</a> before enabling this feature.</p>

						<div style="border-bottom:1px solid #ccc;">
							<h3>Enable SMS Verification When An Order Is Flagged For Review</h3>
						</div>

						<p>Trigger SMS verification when an order was flagged for <strong>REVIEW</strong> by FraudLabs Pro which will happen after the checkout process. The system will email the user providing them with the link to complete the SMS verification process.<br /><br /><i>Please note that you have to install <a href="https://wordpress.org/plugins/fraudlabs-pro-for-easy-digital-downloads/" target="_blank">FraudLabs Pro for Easy Digital Downloads plugin</a> before enabling this feature.</i></p>

						<form method="post">
							<table class="form-table">
								<tr>
									<th scope="row" style="padding-left:35px;">
										<label for="edd_email_verify">Enable</label>
									</th>
									<td>
										<input name="edd_email_verify" type="checkbox" id="edd_email_verify"' . ((get_option('fraudlabs_pro_sms_verification_edd_email_verify')) ? ' checked = "checked"' : '') . ' />
									</td>
								</tr>
								<tr>
									<th scope="row" style="padding-left:35px;"><label for="edd_email_subject">Email Subject</label></th>
									<td>
										<input name="edd_email_subject" type="text" id="edd_email_subject" value="' . get_option('fraudlabs_pro_sms_verification_edd_email_subject') . '" class="regular-text" />
										<p class="description">Subject or title of the mail.</p>
									</td>
								</tr>
								<tr>
									<th scope="row" style="padding-left:35px;"><label for="edd_email_content">Email Body</label></th>
									<td>
										<textarea id="edd_email_content" name="edd_email_content" rows="4" cols="50">' . get_option('fraudlabs_pro_sms_verification_edd_email_content') . '</textarea>
										<p class="description">Email body. You must include the {email_verification_link} tag which will be automatically replaced with the link for the SMS verification process.</p>
									</td>
								</tr>
							</table>

							<input type="hidden" name="form_edd_submitted" value="" />
							<input type="submit" name="submit" id="submit" class="button button-primary" value="Save Changes" />
						</form>
					</div>
				</div>';
				break;

			// Shortcodes tab.
			case 'shortcodes':
				echo '
				<div class="wrap">
					<h1>FraudLabs Pro SMS Verification</h1>
					<p>
						This plugin helps you to verify the user\'s mobile phone number by using the SMS verification technology. It sends an OTP (One Time Passcode) to the user\'s given phone number, and then this OTP shall be entered into the form to confirm the phone ownership.
					</p>

					' . $this->admin_tabs() . '
					<div style="margin-left:20px;">
						<p>
							<div style="margin-bottom:40px;">
								<h3>Shortcode for WordPress</h3>
								<pre> [flp_sms_verification] </pre>
								<p>Please embed this shortcode into the post, article or page to enable the SMS verification.<p>
							</div>
							<div>
								<h3>Shortcode for Contact Form 7</h3>
								<pre> [flp_sms_verification] </pre>
								<p>Please embed this shortcode into the Contact Form to enable the SMS verification.<p>
								<pre> [hidden flp-sms-phone class:flp-sms] </pre>
								<p>You can embed this shortcode into the Contact Form to get the verified phone number.<p>
							</div>
						</p>

					</div>
				</div>';
				break;

			// Settings tab.
			case 'settings':
			default:
				$sms_credit_available = $sms_date_expired = '';
				// Use plan API to get license information.
				$request = wp_remote_get('https://api.fraudlabspro.com/v1/plan?' . http_build_query(array(
					'key'		=> $this->api_key,
					'format'	=> 'json'
				)));

				if (!is_wp_error($request)) {
					// Get the HTTP response.
					$response = json_decode(wp_remote_retrieve_body($request));

					if (is_object($response)) {
						$sms_credit_available = $response->sms_available;
						$sms_date_expired = $response->sms_date_expired;
					}
				}

				$apiKey = (isset($_POST['api_key'])) ? sanitize_text_field($_POST['api_key']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_api_key') : '');
				$smsRetries = (isset($_POST['sms_retries'])) ? sanitize_text_field($_POST['sms_retries']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_sms_retries') : '');
				$smsInstruction = (isset($_POST['sms_instruction'])) ? sanitize_text_field($_POST['sms_instruction']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_sms_instruction') : '');
				$smsTemplate = (isset($_POST['sms_template'])) ? sanitize_text_field($_POST['sms_template']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_sms_template') : '');
				$otpTimeout = (isset($_POST['otp_timeout'])) ? sanitize_text_field($_POST['otp_timeout']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_otp_timeout') : '3600');
				$sms_tel_cc = (isset($_POST['sms_tel_cc'])) ? sanitize_text_field($_POST['sms_tel_cc']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_sms_tel_cc') : 'US');
				$enableDebugLog = (isset($_POST['enable_debug_log'])) ? 1 : 0;
				$msgSmsRequired = (isset($_POST['msg_sms_required'])) ? sanitize_text_field($_POST['msg_sms_required']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_msg_sms_required') : 'You are required to verify you phone number via SMS verification.');
				$msgOtpSuccess = (isset($_POST['msg_otp_success'])) ? sanitize_text_field($_POST['msg_otp_success']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_msg_otp_success') : 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.');
				$msgOtpFail = (isset($_POST['msg_otp_fail'])) ? sanitize_text_field($_POST['msg_otp_fail']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_msg_otp_fail') : 'Error: Unable to send the SMS verification message to {phone}.');
				$msgInvalidPhone = (isset($_POST['msg_invalid_phone'])) ? sanitize_text_field($_POST['msg_invalid_phone']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') : 'Please enter a valid phone number.');
				$msgInvalidOtp = (isset($_POST['msg_invalid_otp'])) ? sanitize_text_field($_POST['msg_invalid_otp']) : (!isset($_POST['submit']) ? get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') : 'Error: Invalid OTP. Please enter the correct OTP.');

			if (isset($_POST['setting_submitted'])) {
				if (strpos($smsTemplate, '{otp}') == false) {
					$status .= '
					<div id="message" class="error">
						<p>The <strong>{otp}</strong> tag must be included in the SMS Template.</p>
					</div>';
				} else if (($msgOtpSuccess != '') && (strpos($msgOtpSuccess, '{phone}') == false)) {
					$status .= '
					<div id="message" class="error">
						<p>The <strong>{phone}</strong> tag must be included in the OTP Sent Succesfully Message.</p>
					</div>';
				} else if (($msgOtpFail != '') && (strpos($msgOtpFail, '{phone}') == false)) {
					$status .= '
					<div id="message" class="error">
						<p>The <strong>{phone}</strong> tag must be included in the OTP Sent Failed Message.</p>
					</div>';
				} else {
					update_option('fraudlabs_pro_sms_verification_api_key', $apiKey);
					update_option('fraudlabs_pro_sms_verification_sms_retries', $smsRetries);
					update_option('fraudlabs_pro_sms_verification_sms_instruction', $smsInstruction);
					update_option('fraudlabs_pro_sms_verification_sms_template', $smsTemplate);
					update_option('fraudlabs_pro_sms_verification_otp_timeout', $otpTimeout);
					update_option('fraudlabs_pro_sms_verification_sms_tel_cc', $sms_tel_cc);
					update_option('fraudlabs_pro_sms_verification_debug_log_enabled', $enableDebugLog);
					update_option('fraudlabs_pro_sms_verification_msg_sms_required', $msgSmsRequired);
					update_option('fraudlabs_pro_sms_verification_msg_otp_success', $msgOtpSuccess);
					update_option('fraudlabs_pro_sms_verification_msg_otp_fail', $msgOtpFail);
					update_option('fraudlabs_pro_sms_verification_msg_invalid_phone', $msgInvalidPhone);
					update_option('fraudlabs_pro_sms_verification_msg_invalid_otp', $msgInvalidOtp);

					if ($apiKey !== $this->api_key) {
						// Use plan API to get license information.
						$request = wp_remote_get('https://api.fraudlabspro.com/v1/plan?' . http_build_query(array(
							'key'		=> $apiKey,
							'format'	=> 'json'
						)));

						if (!is_wp_error($request)) {
							// Get the HTTP response.
							$response = json_decode(wp_remote_retrieve_body($request));

							if (is_object($response)) {
								$sms_credit_available = $response->sms_available;
								$sms_date_expired = $response->sms_date_expired;
							}
						}
					}

					$status .= '
					<div id="message" class="updated">
						<p>The changes have been successfully saved.</p>
					</div>';
				}
			}

			echo '
			<div class="wrap">
				<h1>FraudLabs Pro SMS Verification</h1>
				<p>
					This plugin helps you to verify the user\'s mobile phone number by using the SMS verification technology. It sends an OTP (One Time Passcode) to the user\'s given phone number, and then this OTP shall be entered into the form to confirm the phone ownership.
				</p>
				<p>
					Please visit <a href="https://www.fraudlabspro.com/developer/reference/country-codes-sms" target="_blank">https://www.fraudlabspro.com/developer/reference/country-codes-sms</a> to learn more about the supported countries.
				</p>

				' . $this->admin_tabs() . '

				' . $status . '
				<div style="margin-left:20px;">
					<form method="post">
						<table class="form-table">
							<tr>
								<th scope="row"><label for="sms_credit_available">SMS Credits</label></th>
								<td>
									<p>' . $sms_credit_available . ' &nbsp;&nbsp;<input type="button" name="sms-top-up" id="sms-top-up" class="button" style="background-color:red; color:white;" value="Add More Credits" /></p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="sms_date_expired">SMS Credits Expiration Date</label></th>
								<td>
									<p>' . $sms_date_expired . ' UTC</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="api_key">API Key</label></th>
								<td>
									<input name="api_key" type="text" id="api_key" value="' . get_option('fraudlabs_pro_sms_verification_api_key') . '" class="regular-text" />
									<p class="description">Please sign up for a free Micro plan at <a href="https://www.fraudlabspro.com/pricing" target="_blank">FraudLabs Pro</a>, if you do not have the API key.</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="sms_retries">Max SMS Retries</label></th>
								<td>
									<select name="sms_retries" id="sms_retries" class="regular-text">
										<option value="0"' . ((get_option('fraudlabs_pro_sms_verification_sms_retries') == 0) ? ' selected' : '') . '>0</option>
										<option value="1"' . ((get_option('fraudlabs_pro_sms_verification_sms_retries') == 1) ? ' selected' : '') . '>1</option>
										<option value="2"' . ((get_option('fraudlabs_pro_sms_verification_sms_retries') == 2) ? ' selected' : '') . '>2</option>
										<option value="3"' . ((get_option('fraudlabs_pro_sms_verification_sms_retries') == 3) ? ' selected' : '') . '>3</option>
										<option value="4"' . ((get_option('fraudlabs_pro_sms_verification_sms_retries') == 4) ? ' selected' : '') . '>4</option>
										<option value="5"' . ((get_option('fraudlabs_pro_sms_verification_sms_retries') == 5) ? ' selected' : '') . '>5</option>
									</select>
									<p class="description">Maximum number of SMS (OTP) retries allowed for resend in a session.</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="sms_instruction">SMS Verification Instruction</label></th>
								<td>
									<textarea id="sms_instruction" name="sms_instruction" rows="4" cols="50">' . get_option('fraudlabs_pro_sms_verification_sms_instruction') . '</textarea>
									<p class="description">Messages to brief the user about this SMS verification and what needs to be done to complete the verification.</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="sms_template">SMS Message Content</label></th>
								<td>
									<input name="sms_template" type="text" id="sms_template" value="' . get_option('fraudlabs_pro_sms_verification_sms_template') . '" class="regular-text" />
									<p class="description">The SMS text message to be sent to the user\'s mobile phone. You must include the {otp} tag which will contain the auto-generated OTP code.</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="otp_timeout">SMS OTP Timeout</label></th>
								<td>
									<input name="otp_timeout" type="text" id="otp_timeout" value="' . get_option('fraudlabs_pro_sms_verification_otp_timeout') . '" class="regular-text" />
									<p class="description">OTP validtity timeout. The default value is 3600 seconds (1 hour) and the maximum value is 86400 seconds (24 hours).</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="sms_tel_cc">Default Country Code For SMS Sending</label></th>
								<td>
									<select name="sms_tel_cc" id="sms_tel_cc" data-placeholder="Choose countries..." class="input-field">';
										foreach ($this->countries as $country_code => $country_name) {
											echo '
												<option value="' . $country_code . '"' . (($sms_tel_cc == $country_code ) ? ' selected' : '') . '> ' . $country_name . '</option>';
										}
									echo'
									</select>
								</td>
							</tr>
							<tr>
								<th scope="row">
									<label for="enable_debug_log">Enable Debug Log</label>
								</th>
								<td>
									<input type="checkbox" name="enable_debug_log" id="enable_debug_log"' . ((get_option('fraudlabs_pro_sms_verification_debug_log_enabled')) ? ' checked = "checked"' : '') . '>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="msg_sms_required">SMS Verification Required Message</label></th>
								<td>
									<textarea id="msg_sms_required" name="msg_sms_required" rows="4" cols="50">' . get_option('fraudlabs_pro_sms_verification_msg_sms_required') . '</textarea>
									<p class="description">Messages to show the user that the phone number verification via SMS verification is required.</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="msg_otp_success">OTP Sent Succesfully Message</label></th>
								<td>
									<textarea id="msg_otp_success" name="msg_otp_success" rows="4" cols="50">' . get_option('fraudlabs_pro_sms_verification_msg_otp_success') . '</textarea>
									<p class="description">Messages to show the user when the OTP is sent successfully to the phone number. You must include the {phone} tag which will be replaced by the user\'s phone number.</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="msg_otp_fail">OTP Sent Failed Message</label></th>
								<td>
									<textarea id="msg_otp_fail" name="msg_otp_fail" rows="4" cols="50">' . get_option('fraudlabs_pro_sms_verification_msg_otp_fail') . '</textarea>
									<p class="description">Messages to show the user when the OTP is sent failed to the phone number. You must include the {phone} tag which will be replaced by the user\'s phone number.</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="msg_invalid_phone">Invalid Phone Number Message</label></th>
								<td>
									<textarea id="msg_invalid_phone" name="msg_invalid_phone" rows="4" cols="50">' . get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') . '</textarea>
									<p class="description">Messages to show the user when invalid phone number is entered.</p>
								</td>
							</tr>
							<tr>
								<th scope="row"><label for="msg_invalid_otp">Invalid OTP Message</label></th>
								<td>
									<textarea id="msg_invalid_otp" name="msg_invalid_otp" rows="4" cols="50">' . get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') . '</textarea>
									<p class="description">Messages to show the user when invalid OTP is entered.</p>
								</td>
							</tr>
						</table>

						<input type="hidden" name="setting_submitted" value="" />
						<input type="submit" name="submit" id="submit" class="button button-primary" value="Save Changes" />
					</form>
				</div>
			</div>';
		}
	}

	/**
	 * Initial default settings.
	 */
	function set_defaults() {
		if (!get_option('fraudlabs_pro_sms_verification_api_key')) update_option('fraudlabs_pro_sms_verification_api_key', '');
		update_option('fraudlabs_pro_sms_verification_sms_retries', '3');
		update_option('fraudlabs_pro_sms_verification_sms_instruction', '');
		update_option('fraudlabs_pro_sms_verification_sms_template', 'Hi, your OTP is {otp}.');
		update_option('fraudlabs_pro_sms_verification_otp_timeout', '3600');
		update_option('fraudlabs_pro_sms_verification_sms_tel_cc', 'US');
		update_option('fraudlabs_pro_sms_verification_debug_log_enabled', 0);
		update_option('fraudlabs_pro_sms_verification_msg_sms_required', 'You are required to verify you phone number via SMS verification.');
		update_option('fraudlabs_pro_sms_verification_msg_otp_success', 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.');
		update_option('fraudlabs_pro_sms_verification_msg_otp_fail', 'Error: Unable to send the SMS verification message to {phone}.');
		update_option('fraudlabs_pro_sms_verification_msg_invalid_phone', 'Please enter a valid phone number.');
		update_option('fraudlabs_pro_sms_verification_msg_invalid_otp', 'Error: Invalid OTP. Please enter the correct OTP.');

		update_option('fraudlabs_pro_sms_verification_wp_default_registration_form', 0);
		update_option('fraudlabs_pro_sms_verification_wp_default_login_form', 0);

		update_option('fraudlabs_pro_sms_verification_wc_default_registration_form', 0);
		update_option('fraudlabs_pro_sms_verification_wc_default_checkout_form', 0);
		update_option('fraudlabs_pro_sms_verification_wc_email_verify', 0);

		update_option('fraudlabs_pro_sms_verification_edd_email_verify', 0);

		$this->callback_external();
		flush_rewrite_rules();
	}

	/**
	 * Set rewrite rule.
	 */
	public function callback_external() {
		global $wp;
		$current_host = isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '';
		$current_uri = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '';
		$current_url = $current_host . $current_uri;

		if (strpos($current_url, 'flp-verify.php')) {
			include(plugin_dir_path(__FILE__) . '/includes/flp-verify.php');
		}
	}

	/**
	 * Shortcode for SMS Verification.
	 */
	public function flp_sms_form() {
		$table_name = $this->create_sms_table();

		$flp_ajax_field = 'flp_ajax_' . substr(hash('sha256', uniqid()), 8, 24); // Dynamic field name for authentication.

		$ip = $this->get_user_ip();

		$smsdata = $this->query_sms_table($table_name, $ip);

		if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
			// Reset counter.
			$this->replace_sms_table($table_name, $ip, $this->sms_retries, time(), $flp_ajax_field);
			$smsdata = $this->query_sms_table($table_name, $ip);
		}
		else {
			// Just update ajax field.
			$this->replace_sms_table($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field);
		}
		// Refresh data.
		$smsdata = $this->query_sms_table($table_name, $ip);

		$retries = $smsdata->counter;
		// $flp_sms_form = '<h3 id="verifysms">SMS Verification <abbr class="required" title="required">*</abbr></h3>';
		$flp_sms_form = '<div id="flp_sms_box" style="font-size: 12px; border: 1px solid silver; padding: 5px;">';
		$flp_sms_form .= '<div id="sms_err" style="background-color:#f8d7da;color:#7d5880;padding:10px;margin-bottom:20px;font-size:1em;display:none;"></div>';
		if (!empty($this->sms_instruction)) $flp_sms_form .= '<span id="sms_info_display">' . nl2br($this->sms_instruction) . ' <br /><br /></span>';
		$flp_sms_form .= '<span id="sms_status">' . (($retries < 0) ? 'Maximum number of retries to send verification SMS exceeded. Please try again later.' : '') . '</span>';
		if (($retries >= 0)) {
			$flp_sms_form .= '<label for="phone_number" id="enter_phone_number" style="font-weight:bold margin-top:10px; margin-bottom:10px;">Please enter Phone Number (with country code)<br /><input type="text" name="phone_number" id="phone_number" value="" style="width:100%;" /></label>';
		}
		$flp_sms_form .= '<br /><div id="sms_otp" style="margin-top: 20px; display: none;"><span style="font-weight:bold;">Please enter OTP (One Time Passcode):</span><br /><input type="text" name="sms_otp1" id="sms_otp1" value="" style="width:130px; background-color:#bbb;">&nbsp;&nbsp;-&nbsp;&nbsp;<input type="text" name="sms_otp2" id="sms_otp2" value="" placeholder="Enter 6-digits OTP"></div>';
		$flp_sms_form .= '<div style="margin-top:20px;"><input type="button" name="flp_submit_otp" id="flp_submit_otp" value="Submit OTP" style="margin-right: 10px; display: none;">';
		$flp_sms_form .= '<input type="button" name="flp_get_otp" id="flp_get_otp" value="Request OTP" style="margin-right: 10px;' . (($retries < 0) ? 'display: none;' : '') . '">';
		$flp_sms_form .= '<input type="button" name="flp_resend_otp" id="flp_resend_otp" value="Resend OTP" style="margin-right: 10px; display: none;">';
		$flp_sms_form .= '<input type="hidden" name="sms_verified" id="sms_verified" value="">';
		$flp_sms_form .= '<input type="hidden" name="sms_tran_id" id="sms_tran_id" value="">';
		$flp_sms_form .= '<input type="hidden" name="sms_phone_cc" id="sms_phone_cc" value="' . get_option('fraudlabs_pro_sms_verification_sms_tel_cc') . '">';
		$flp_sms_form .= '<input type="hidden" name="sms_retries" id="sms_retries" value="' . $retries . '">';
		$flp_sms_form .= '</div></div>';
		$flp_sms_form .= '<div id="flp_sms_success_status" style="border: 1px solid silver; padding: 10px; background-color: #22B14C; color: white; display: none;"><span>Your phone number has been successfully verified.</span></div><br />';

		$msgOtpSuccess = (get_option('fraudlabs_pro_sms_verification_msg_otp_success') == '') ? 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.' : get_option('fraudlabs_pro_sms_verification_msg_otp_success');
		$msgOtpSuccess = explode("{phone}", $msgOtpSuccess);
		$msgOtpFail = (get_option('fraudlabs_pro_sms_verification_msg_otp_fail') == '') ? 'Error: Unable to send the SMS verification message to {phone}.' : get_option('fraudlabs_pro_sms_verification_msg_otp_fail');
		$msgOtpFail = explode("{phone}", $msgOtpFail);
		$msgInvalidPhone = (get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') == '') ? 'Please enter a valid phone number.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_phone');
		$msgInvalidOtp = (get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') == '') ? 'Error: Invalid OTP. Please enter the correct OTP.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_otp');
		$flp_sms_form .= '
		<style>
			input[type=text]:disabled {
				background: #dddddd;
			}
		</style>
		<script src="//ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>
		<script src="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/js/intlTelInput.min.js"></script>
		<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/css/intlTelInput.min.css">
		<script language="Javascript">
			var phoneNum;
			jQuery(document).ready(function() {
				phoneNum = window.intlTelInput(document.querySelector("#phone_number"), {
					utilsScript: "https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/js/utils.min.js",
					separateDialCode: true,
					initialCountry: jQuery("#sms_phone_cc").val()
				});
			});
			function doOTP() {
				jQuery("input.flp-sms").val(phoneNum.getNumber());
				var data = {
					"action": "wp_sms_action_send",
					"tel": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success,
					error: sms_error,
					dataType: "text"
				});
			}
			function checkOTP() {
				var data = {
					"action": "wp_sms_action_verify",
					"otp": jQuery("#sms_otp1").val() + "-" + jQuery("#sms_otp2").val(),
					"tran_id": jQuery("#sms_tran_id").val()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success2,
					error: sms_error2,
					dataType: "text"
				});
			}
			jQuery("#sms_otp2").bind("keypress", function(e) {
				var code = e.keyCode || e.which;
				if (code == 13) { //Enter keycode
					e.preventDefault();
				}
			});
			jQuery("#flp_get_otp").click(function(e) {
				if (jQuery("#phone_number").val() == "") {
					jQuery("#sms_err").html("'; $flp_sms_form .= $msgInvalidPhone; $flp_sms_form .= '");
					jQuery("#sms_err").show();
					jQuery("#phone_number").focus();
				} else if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_resend_otp").click(function(e) {
				if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_submit_otp").click(function() {
				checkOTP();
			});
			jQuery(\'form\').submit(function(event) {
				if (jQuery.trim(jQuery("#sms_verified").val()) == "") {
					// not yet verified
					event.preventDefault();
					event.stopImmediatePropagation();
					alert("Please complete the SMS Verification.");
					jQuery("#sms_otp").focus();
					document.getElementById("verifysms").scrollIntoView(true);
				}
			});
			function sms_success(data) {
				// need to count down the number of retries
				jQuery("#sms_retries").val(jQuery("#sms_retries").val() - 1);
				if (data.indexOf("ERROR") == 0) {
					jQuery("#sms_err").html("'; $flp_sms_form .= $msgOtpFail[0]; $flp_sms_form .='" + phoneNum.getNumber() + "'; $flp_sms_form .= $msgOtpFail[1]; $flp_sms_form .='");
					jQuery("#sms_err").show();
					if (jQuery("#sms_retries").val() == 0) {
						jQuery("#enter_phone_number").hide();
						jQuery("#sms_otp").hide();
						jQuery("#flp_submit_otp").hide();
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
						jQuery("#sms_status").text("Maximum number of retries to send verification SMS exceeded. Please try again later.");
					}
				} else if (data.indexOf("OK") == 0) {
					alert("'; $flp_sms_form .= $msgOtpSuccess[0]; $flp_sms_form .='" + phoneNum.getNumber() + "'; $flp_sms_form .= $msgOtpSuccess[1]; $flp_sms_form .='");
					// store the tran_id
					jQuery("#sms_tran_id").val(data.substr(2,20));
					// show the submit otp button
					jQuery("#flp_get_otp").hide();
					jQuery("#sms_err").hide();
					jQuery("#flp_resend_otp").show();
					jQuery("#flp_submit_otp").show();
					jQuery("#sms_otp").show();
					jQuery("#sms_otp1").val(data.substr(22,6));
					jQuery("#phone_number").prop("disabled", true);
					jQuery("#sms_otp1").prop("disabled", true);
					if (jQuery("#sms_retries").val() == 0) {
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
					}
				}
			}
			function sms_error() {
				jQuery("#sms_err").html("'; $flp_sms_form .= $msgOtpFail[0]; $flp_sms_form .='" + phoneNum.getNumber() + "'; $flp_sms_form .= $msgOtpFail[1]; $flp_sms_form .='");
				jQuery("#sms_err").show();
			}
			function sms_success2(data) {
				if (data.indexOf("ERROR 601") == 0) {
					jQuery("#sms_err").html("'; $flp_sms_form .= $msgInvalidOtp; $flp_sms_form .= '");
					jQuery("#sms_err").show();
				} else if (data.indexOf("ERROR 600") == 0) {
					jQuery("#sms_err").html("Error: Error while performing verification.");
					jQuery("#sms_err").show();
				} else if (data.indexOf("OK") == 0) {
					// update the sms verified field
					jQuery("#sms_verified").val("YES");
					// hide all buttons and text field for the SMS...
					jQuery("#sms_otp").hide();
					jQuery("#flp_submit_otp").hide();
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").hide();
					jQuery("#sms_err").hide();
					// show SMS verified message
					jQuery("#flp_sms_box").hide();
					jQuery("#flp_sms_success_status").show();
				}
			}
			function sms_error2() {
				jQuery("#sms_err").html("Error: Could not perform SMS verification.");
				jQuery("#sms_err").show();
			}
		</script>
		';
		return $flp_sms_form;
	}

	/**
	 * Add sms verification to WP default registration form.
	 */
	public function wp_default_register_form() {
		if ((get_option('fraudlabs_pro_sms_verification_wp_default_registration_form') == 0)) {
			$this->write_debug_log('WordPress Default Registration Form not selected. SMS Verification will not be performed.');
			return;
		}
		elseif ((trim($this->api_key) == '')) {
			$this->write_debug_log('FraudLabs Pro API Key not inserted. SMS Verification will not be performed.');
			return;
		}

		$table_name = $this->create_sms_table();

		$flp_ajax_field = 'flp_ajax_' . substr(hash('sha256', uniqid()), 8, 24); // dynamic field name for authentication

		$ip = $this->get_user_ip();

		if (isset($_POST["tel_num"])) {
			$pho = sanitize_text_field($_POST["tel_num"]);

			$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);

			if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
				// reset counter
				$this->replace_sms_table_tel($table_name, $ip, $this->sms_retries, time(), $flp_ajax_field, $pho);
				$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);
			}
			else {
				// Just update ajax field.
				$this->replace_sms_table_tel($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field, $pho);
			}
			// Refresh data.
			$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);
		} else {
			$smsdata = $this->query_sms_table($table_name, $ip);

			if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
				// Reset counter.
				$this->replace_sms_table($table_name, $ip, $this->sms_retries, time(), $flp_ajax_field);
				$smsdata = $this->query_sms_table($table_name, $ip);
			}
			else {
				// Just update ajax field.
				$this->replace_sms_table($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field);
			}
			// Refresh data.
			$smsdata = $this->query_sms_table($table_name, $ip);
		}

		$retries = $smsdata->counter;
		$msgSmsRequired = (get_option('fraudlabs_pro_sms_verification_msg_sms_required') == '') ? 'You are required to verify you phone number via SMS verification.' : get_option('fraudlabs_pro_sms_verification_msg_sms_required');
		echo '<div id="flp-sms-verification" class="mfp-hide white-popup">';
		echo '<div id="flp_sms" style="padding:23px;text-align:center;"><img src="' . plugins_url('/assets/images/flp-get-otp.png', __FILE__) . '" style="width:130px;margin-bottom:20px;"><br /><h3 id="verifysms" style="margin-bottom:10px;">Verify Phone Number <small>(via SMS Verification)</small></h3>';
		echo '<span id="sms_status">' . (($retries < 0) ? "Maximum number of retries to send verification SMS exceeded. Please try again later.<br /><br />" : '') . '</span>';
		echo '<div id="sms_err" style="background-color:#f8d7da;color:#7d5880;padding:10px;margin-bottom:10px;font-size:1em;display:none;"></div>';
		if (($retries >= 0)) {
			if (!empty($this->sms_instruction)) echo '<span id="sms_info_display" style="margin-bottom:10px;">' . nl2br($this->sms_instruction) . ' <br /><br /></span>';
			echo '<div><label for="phone_number" id="enter_phone_number" style="margin-top: 15px; margin-bottom: 15px;"><input type="text" name="phone_number" id="phone_number" value="" style="font-size:19px;"/></label></div>';
		}
		echo '<span id="sms_phone_info" style="display:none;"></span>';
		echo '<div id="sms_otp" style="margin-top:5px;display: none;"><input type="text" name="sms_otp1" id="sms_otp1" value="" style="width:130px;font-size:14px;padding:10px;">&nbsp;&nbsp;-&nbsp;&nbsp;<input type="text" name="sms_otp2" id="sms_otp2" value="" placeholder="Enter OTP numbers" style="font-size:14px;padding:10px;width:150px;"></div>';
		echo '<input type="button" class="button alt" name="flp_submit_otp" id="flp_submit_otp" value="Verify" style="width:55%;font-size:14px;padding:6px 12px;display: none;">';
		echo '<input type="button" class="button alt" name="flp_get_otp" id="flp_get_otp" value="Request OTP" style="padding:6px 12px;width:55%;' . (($retries < 0) ? 'display: none;' : '') . '">';
		echo '<div name="flp_resend_otp" id="flp_resend_otp" style="display:none; margin-top:10px;">Didn\'t receive OTP? <a href="javascript:void(0);" ><u>Resend OTP</u></a></div>';
		echo '<input type="hidden" name="sms_verified" id="sms_verified" value="">';
		echo '<input type="hidden" name="sms_tran_id" id="sms_tran_id" value="">';
		echo '<input type="hidden" name="sms_phone_cc" id="sms_phone_cc" value="' . get_option('fraudlabs_pro_sms_verification_sms_tel_cc') . '">';
		echo '<input type="hidden" name="sms_retries" id="sms_retries" value="' . $retries . '">';
		echo '</div>';
		echo '<div id="flp_sms_success_status" style="padding: 20px;text-align:center; display: none;"><img src="' . plugins_url('/assets/images/flp-success-icon.png', __FILE__) . '" style="width:160px;"><br /><h3 style="margin-bottom:10px;">Your phone number has been successfully verified.</h3></div>';
		echo '</div>';

		echo '<div id="flp_sms_inline" style="background:#fff;box-shadow: 0 5px 11px 0 rgba(0,0,0,0.18), 0 4px 15px 0 rgba(0,0,0,0.15);padding:1.1rem;"><h3 id="verifysms">SMS Verification</h3>';
		echo '<div id="flp_sms_box" style="font-size: 15px; padding: 5px;">';
		echo "<div id='flp_box_link'><span>" . $msgSmsRequired . "</span><br /><br />";
		echo '<input type="button" name="verify_flp_sms_inline" id="verify_flp_sms_inline" data-mfp-src="#flp-sms-verification" class="open-popup" href="javascript:;" class="button" value="Verify Your Phone Number Now" style="padding:7px 14px;font-size:13px;' . (($retries < 0) ? 'display: none;' : '') . '"></div></div>';
		echo '<div id="flp_box_success" style="display:none; padding: 5px;margin-top:10px;margin-bottom:10px;"><img src="' . plugins_url('/assets/images/tick.png', __FILE__) . '" style="display: block; margin-left: auto; margin-right: auto;"></div>';
		echo '</div><br />';

		$msgOtpSuccess = (get_option('fraudlabs_pro_sms_verification_msg_otp_success') == '') ? 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.' : get_option('fraudlabs_pro_sms_verification_msg_otp_success');
		$msgOtpSuccess = explode("{phone}", $msgOtpSuccess);
		$msgOtpFail = (get_option('fraudlabs_pro_sms_verification_msg_otp_fail') == '') ? 'Error: Unable to send the SMS verification message to {phone}.' : get_option('fraudlabs_pro_sms_verification_msg_otp_fail');
		$msgOtpFail = explode("{phone}", $msgOtpFail);
		$msgInvalidPhone = (get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') == '') ? 'Please enter a valid phone number.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_phone');
		$msgInvalidOtp = (get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') == '') ? 'Error: Invalid OTP. Please enter the correct OTP.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_otp');
		echo '
		<script src="//ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>

		<script src="https://cdnjs.cloudflare.com/ajax/libs/magnific-popup.js/1.1.0/jquery.magnific-popup.min.js"></script>
		<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/magnific-popup.js/1.1.0/magnific-popup.min.css"/>

		<script src="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/js/intlTelInput.min.js"></script>
		<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/css/intlTelInput.min.css">

		<style>
			.white-popup {position: relative;background: #FFF;	padding: 22px;	width:auto;	max-width: 600px; height:px; margin: 20px auto; overflow-y:auto;}
			@media screen and (max-width: 280px) {.mfp-bg, .mfp-ready {position:fixed ! important;}}
			@media (width: 1024px) {.mfp-container{top:-250px ! important;}}
			@media (width: 768px) {.mfp-container{top:-100px ! important;}}
		</style>

		<script language="Javascript">
			var phoneNum;
			jQuery(document).ready(function() {
				phoneNum = window.intlTelInput(document.querySelector("#phone_number"), {
					utilsScript: "https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/js/utils.min.js",
					separateDialCode: true,
					initialCountry: jQuery("#sms_phone_cc").val()
				});

				jQuery(".open-popup").magnificPopup({
					type:"inline",
					midClick: true
				});
			});
			function getPhoneNum() {
				var data = {
					"tel_num": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					data: data,
					dataType: "text"
				});
			}
			function doOTP() {
				var data = {
					"action": "wp_sms_action_send",
					"tel": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success,
					error: sms_error,
					dataType: "text"
				});
			}
			function checkOTP() {
				var data = {
					"action": "wp_sms_action_verify",
					"otp": jQuery("#sms_otp1").val() + "-" + jQuery("#sms_otp2").val(),
					"tran_id": jQuery("#sms_tran_id").val()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success2,
					error: sms_error2,
					dataType: "text"
				});
			}
			jQuery("#sms_otp2").bind("keypress", function(e) {
				var code = e.keyCode || e.which;
				if (code == 13) { //Enter keycode
					e.preventDefault();
				}
			});
			jQuery("#flp_get_otp").click(function(e) {
				if (jQuery("#phone_number").val() == "") {
					jQuery("#sms_err").html("'; echo $msgInvalidPhone; echo'");
					jQuery("#sms_err").show();
					jQuery("#phone_number").focus();
				} else if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					getPhoneNum();
					doOTP();
				}
			});
			jQuery("#flp_resend_otp").click(function(e) {
				if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_submit_otp").click(function() {
				checkOTP();
			});
			jQuery(\'form[name="registerform"]\').submit(function(event) {
				if (jQuery.trim(jQuery("#sms_verified").val()) == "") {
					// not yet verified
					event.preventDefault();
					event.stopImmediatePropagation();
					alert("Please complete the SMS Verification.");
					jQuery("#sms_otp1").focus();
					document.getElementById("verifysms").scrollIntoView(true);
				}
			});
			function sms_success(data) {
				// need to count down the number of retries
				jQuery("#sms_retries").val(jQuery("#sms_retries").val() - 1);
				if (data.indexOf("ERROR") == 0) {
					jQuery("#sms_err").html("'; echo $msgOtpFail[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpFail[1]; echo'");
					jQuery("#sms_err").show();
					if (data.substr(10,1) == "-") {
						jQuery("#sms_otp").hide();
						jQuery("#flp_submit_otp").hide();
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
						jQuery("#sms_status").html("Maximum number of retries to send verification SMS exceeded. Please try again later.");
					}
				} else if (data.indexOf("OK") == 0) {
					// store the tran_id
					jQuery("#sms_tran_id").val(data.substr(2,20));
					// show the submit otp button
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").show();
					jQuery("#flp_submit_otp").show();
					jQuery("#sms_otp").show();
					jQuery("#sms_err").hide();
					jQuery("#sms_phone_info").html("'; echo $msgOtpSuccess[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpSuccess[1]; echo'<br />");
					jQuery("#sms_phone_info").show();
					jQuery("#sms_info_display").hide();
					jQuery("#enter_phone_number").hide();
					jQuery("#sms_err").hide();
					jQuery("#sms_otp1").val(data.substr(22,6));
					jQuery("#sms_otp1").prop("disabled", true);
					if (data.substr(28,1) == "-") {
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
					}
				}
			}
			function sms_error() {
				jQuery("#sms_err").html("'; echo $msgOtpFail[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpFail[1]; echo'");
				jQuery("#sms_err").show();
			}
			function sms_success2(data) {
				if (data.indexOf("ERROR 601") == 0) {
					jQuery("#sms_err").html("'; echo $msgInvalidOtp; echo '");
					jQuery("#sms_err").show();
				} else if (data.indexOf("ERROR 600") == 0) {
					jQuery("#sms_err").html("Error: Error while performing verification.");
					jQuery("#sms_err").show();
				} else if (data.indexOf("OK") == 0) {
					// update the sms verified field
					jQuery("#sms_verified").val("YES");
					if(typeof(Storage) !== "undefined") {
						sessionStorage.wp_register_sms_vrf = "YES";
					}
					// hide all buttons and text field for the SMS...
					jQuery("#sms_otp").hide();
					jQuery("#flp_submit_otp").hide();
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").hide();
					jQuery("#sms_phone_info").hide();
					jQuery("#sms_err").hide();
					// show SMS verified message
					jQuery("#flp_sms_box").hide();
					jQuery("#flp_sms").hide();
					jQuery("#flp_sms_success_status").show();
					jQuery("#flp_box_link").hide();
					jQuery("#flp_box_success").show();
				}
			}
			function sms_error2() {
				jQuery("#sms_err").html("Error: Could not perform SMS verification.");
				jQuery("#sms_err").show();
			}
			if(sessionStorage.wp_register_sms_vrf == "YES"){
				jQuery("#sms_verified").val("YES");
				jQuery("#sms_otp").hide();
				jQuery("#flp_submit_otp").hide();
				jQuery("#flp_get_otp").hide();
				jQuery("#flp_resend_otp").hide();
				jQuery("#reset_pn").hide();
				jQuery("#sms_phone_info").hide();
				jQuery("#sms_err").hide();
				jQuery("#flp_sms_box").hide();
				jQuery("#flp_sms").hide();
				jQuery("#flp_sms_success_status").show();
				jQuery("#flp_box_link").hide();
				jQuery("#flp_box_success").show();
			}
		</script>
		';
	}

	/**
	 * Add sms verification to WP default login form.
	 */
	function wp_default_login_form() {
		if ((get_option('fraudlabs_pro_sms_verification_wp_default_login_form') == 0)) {
			$this->write_debug_log('WordPress Default Login Form not selected. SMS Verification will not be performed.');
			return;
		}
		elseif ((trim($this->api_key) == '')) {
			$this->write_debug_log('FraudLabs Pro API Key not inserted. SMS Verification will not be performed.');
			return;
		}

		$table_name = $this->create_sms_table();

		$flp_ajax_field = 'flp_ajax_' . substr(hash('sha256', uniqid()), 8, 24); // dynamic field name for authentication

		$ip = $this->get_user_ip();

		if (isset($_POST["tel_num"])) {
			$pho = sanitize_text_field($_POST["tel_num"]);

			$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);

			if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
				// Reset counter.
				$this->replace_sms_table_tel($table_name, $ip, $this->sms_retries, time(), $flp_ajax_field, $pho);
				$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);
			}
			else {
				// Just update ajax field.
				$this->replace_sms_table_tel($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field, $pho);
			}
			// Refresh data.
			$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);
		} else {
			$smsdata = $this->query_sms_table($table_name, $ip);

			if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
				// Reset counter.
				$this->replace_sms_table($table_name, $ip, $this->sms_retries, time(), $flp_ajax_field);
				$smsdata = $this->query_sms_table($table_name, $ip);
			}
			else {
				// Just update ajax field.
				$this->replace_sms_table($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field);
			}
			// Refresh data.
			$smsdata = $this->query_sms_table($table_name, $ip);
		}

		$retries = $smsdata->counter;
		$msgSmsRequired = (get_option('fraudlabs_pro_sms_verification_msg_sms_required') == '') ? 'You are required to verify you phone number via SMS verification.' : get_option('fraudlabs_pro_sms_verification_msg_sms_required');
		echo '<div id="flp-sms-verification" class="mfp-hide white-popup">';
		echo '<div id="flp_sms" style="padding:23px;text-align:center;"><img src="' . plugins_url('/assets/images/flp-get-otp.png', __FILE__) . '" style="width:130px;margin-bottom:20px;"><br /><h3 id="verifysms" style="margin-bottom:10px;">Verify Phone Number <small>(via SMS Verification)</small></h3>';
		echo '<span id="sms_status">' . (($retries < 0) ? "Maximum number of retries to send verification SMS exceeded. Please try again later.<br /><br />" : '') . '</span>';
		echo '<div id="sms_err" style="background-color:#f8d7da;color:#7d5880;padding:10px;margin-bottom:10px;font-size:1em;display:none;"></div>';
		if (($retries >= 0)) {
			if (!empty($this->sms_instruction)) echo '<span id="sms_info_display">' . nl2br($this->sms_instruction) . ' <br /><br /></span>';
			echo '<div><label for="phone_number" id="enter_phone_number" style="margin-top: 15px; margin-bottom: 15px;"><input type="text" name="phone_number" id="phone_number" value="" style="font-size:19px;"/></label></div>';
		}
		echo '<span id="sms_phone_info" style="display:none;"></span>';
		echo '<div id="sms_otp" style="margin-top:5px;display: none;"><input type="text" name="sms_otp1" id="sms_otp1" value="" style="width:130px;font-size:14px;padding:10px;">&nbsp;&nbsp;-&nbsp;&nbsp;<input type="text" name="sms_otp2" id="sms_otp2" value="" placeholder="Enter OTP numbers" style="font-size:14px;padding:10px;width:150px;"></div>';
		echo '<input type="button" class="button alt" name="flp_submit_otp" id="flp_submit_otp" value="Verify" style="width:55%;font-size:14px;padding:6px 12px;display: none;">';
		echo '<input type="button" class="button alt" name="flp_get_otp" id="flp_get_otp" value="Request OTP" style="padding:6px 12px;width:55%;' . (($retries < 0) ? 'display: none;' : '') . '">';
		echo '<div name="flp_resend_otp" id="flp_resend_otp" style="display: none; margin-top:10px;">Didn\'t receive OTP? <a href="javascript:void(0);" ><u>Resend OTP</u></a></div>';
		echo '<input type="hidden" name="sms_verified" id="sms_verified" value="">';
		echo '<input type="hidden" name="sms_tran_id" id="sms_tran_id" value="">';
		echo '<input type="hidden" name="sms_phone_cc" id="sms_phone_cc" value="' . get_option('fraudlabs_pro_sms_verification_sms_tel_cc') . '">';
		echo '<input type="hidden" name="sms_retries" id="sms_retries" value="' . $retries . '">';
		echo '</div>';
		echo '<div id="flp_sms_success_status" style="padding: 20px;text-align:center; display: none;"><img src="' . plugins_url('/assets/images/flp-success-icon.png', __FILE__) . '" style="width:160px;"><br /><h3 style="margin-bottom:10px;">Your phone number has been successfully verified.</h3></div>';
		echo '</div>';

		echo '<div id="flp_sms_inline" style="background:#fff;box-shadow: 0 5px 11px 0 rgba(0,0,0,0.18), 0 4px 15px 0 rgba(0,0,0,0.15);padding:1.1rem;"><h3 id="verifysms">SMS Verification</h3>';
		echo '<div id="flp_sms_box" style="font-size: 15px; padding: 5px;">';
		echo "<div id='flp_box_link'><span>" . $msgSmsRequired . "</span><br /><br />";
		echo '<input type="button" name="verify_flp_sms_inline" id="verify_flp_sms_inline" data-mfp-src="#flp-sms-verification" class="open-popup" href="javascript:;" class="button" value="Verify Your Phone Number Now" style="padding:7px 14px;font-size:13px;' . (($retries < 0) ? 'display: none;' : '') . '"></div></div>';
		echo '<div id="flp_box_success" style="display:none; padding: 5px;margin-top:10px;margin-bottom:10px;"><img src="' . plugins_url('/assets/images/tick.png', __FILE__) . '" style="display: block; margin-left: auto; margin-right: auto;"></div>';
		echo '</div><br />';

		$msgOtpSuccess = (get_option('fraudlabs_pro_sms_verification_msg_otp_success') == '') ? 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.' : get_option('fraudlabs_pro_sms_verification_msg_otp_success');
		$msgOtpSuccess = explode("{phone}", $msgOtpSuccess);
		$msgOtpFail = (get_option('fraudlabs_pro_sms_verification_msg_otp_fail') == '') ? 'Error: Unable to send the SMS verification message to {phone}.' : get_option('fraudlabs_pro_sms_verification_msg_otp_fail');
		$msgOtpFail = explode("{phone}", $msgOtpFail);
		$msgInvalidPhone = (get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') == '') ? 'Please enter a valid phone number.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_phone');
		$msgInvalidOtp = (get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') == '') ? 'Error: Invalid OTP. Please enter the correct OTP.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_otp');
		echo '
		<script src="//ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>

		<script src="https://cdnjs.cloudflare.com/ajax/libs/magnific-popup.js/1.1.0/jquery.magnific-popup.min.js"></script>
		<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/magnific-popup.js/1.1.0/magnific-popup.min.css"/>

		<script src="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/js/intlTelInput.min.js"></script>
		<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/css/intlTelInput.min.css">

		<style>
			.white-popup {position: relative;background: #FFF;	padding: 22px;	width:auto;	max-width: 600px; height:; margin: 20px auto; overflow-y:auto;}
			@media screen and (max-width: 280px) {.mfp-bg, .mfp-ready {position:fixed ! important;}}
			@media (width: 1024px) {.mfp-container{top:-250px ! important;}}
			@media (width: 768px) {.mfp-container{top:-100px ! important;}}
		</style>

		<script language="Javascript">
			var phoneNum;
			jQuery(document).ready(function() {
				phoneNum = window.intlTelInput(document.querySelector("#phone_number"), {
					utilsScript: "https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/js/utils.min.js",
					separateDialCode: true,
					initialCountry: jQuery("#sms_phone_cc").val()
				});

				$(".open-popup").magnificPopup({
					type:"inline",
					midClick: true
				});
			});
			function getPhoneNum() {
				var data = {
					"tel_num": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					data: data,
					dataType: "text"
				});
			}
			function doOTP() {
				var data = {
					"action": "wp_sms_action_send",
					"tel": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success,
					error: sms_error,
					dataType: "text"
				});
			}
			function checkOTP() {
				var data = {
					"action": "wp_sms_action_verify",
					"otp": jQuery("#sms_otp1").val() + "-" + jQuery("#sms_otp2").val(),
					"tran_id": jQuery("#sms_tran_id").val()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success2,
					error: sms_error2,
					dataType: "text"
				});
			}
			jQuery("#sms_otp2").bind("keypress", function(e) {
				var code = e.keyCode || e.which;
				if (code == 13) { //Enter keycode
					e.preventDefault();
				}
			});
			jQuery("#flp_get_otp").click(function(e) {
				if (jQuery("#phone_number").val() == "") {
					jQuery("#sms_err").html("'; echo $msgInvalidPhone; echo'");
					jQuery("#sms_err").show();
					jQuery("#phone_number").focus();
				} else if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					getPhoneNum();
					doOTP();
				}
			});
			jQuery("#flp_resend_otp").click(function() {
				if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_submit_otp").click(function() {
				checkOTP();
			});
			jQuery(\'form[name="loginform"]\').submit(function(event) {
				if (jQuery.trim(jQuery("#sms_verified").val()) == "") {
					// not yet verified
					event.preventDefault();
					event.stopImmediatePropagation();
					alert("Please complete the SMS Verification.");
					jQuery("#sms_otp1").focus();
					document.getElementById("verifysms").scrollIntoView(true);
				}
			});
			function sms_success(data) {
				// need to count down the number of retries
				jQuery("#sms_retries").val(jQuery("#sms_retries").val() - 1);
				if (data.indexOf("ERROR") == 0) {
					jQuery("#sms_err").html("'; echo $msgOtpFail[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpFail[1]; echo'");
					jQuery("#sms_err").show();
					if (data.substr(10,1) == "-") {
						jQuery("#sms_otp").hide();
						jQuery("#flp_submit_otp").hide();
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
						jQuery("#sms_status").html("Maximum number of retries to send verification SMS exceeded. Please try again later.<br /><br />");
					}
				} else if (data.indexOf("OK") == 0) {
					// store the tran_id
					jQuery("#sms_tran_id").val(data.substr(2,20));
					// show the submit otp button
					jQuery("#flp_get_otp").hide();
					jQuery("#sms_err").hide();
					jQuery("#flp_resend_otp").show();
					jQuery("#flp_submit_otp").show();
					jQuery("#sms_otp").show();
					jQuery("#sms_phone_info").html("'; echo $msgOtpSuccess[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpSuccess[1]; echo'<br />");
					jQuery("#sms_phone_info").show();
					jQuery("#sms_info_display").hide();
					jQuery("#enter_phone_number").hide();
					jQuery("#sms_err").hide();
					jQuery("#sms_otp1").val(data.substr(22,6));
					jQuery("#sms_otp1").prop("disabled", true);
					if (data.substr(28,1) == "-") {
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
					}
				}
			}
			function sms_error() {
				jQuery("#sms_err").html("'; echo $msgOtpFail[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpFail[1]; echo'");
				jQuery("#sms_err").show();
			}
			function sms_success2(data) {
				if (data.indexOf("ERROR 601") == 0) {
					jQuery("#sms_err").html("'; echo $msgInvalidOtp; echo '");
					jQuery("#sms_err").show();
				} else if (data.indexOf("ERROR 600") == 0) {
					jQuery("#sms_err").html("Error: Error while performing verification.");
					jQuery("#sms_err").show();
				} else if (data.indexOf("OK") == 0) {
					// update the sms verified field
					jQuery("#sms_verified").val("YES");
					if(typeof(Storage) !== "undefined") {
						sessionStorage.wp_login_sms_vrf = "YES";
					}
					// hide all buttons and text field for the SMS...
					jQuery("#sms_otp").hide();
					jQuery("#flp_submit_otp").hide();
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").hide();
					jQuery("#sms_phone_info").hide();
					jQuery("#sms_err").hide();
					// show SMS verified message
					jQuery("#flp_sms_box").hide();
					jQuery("#flp_sms").hide();
					jQuery("#flp_sms_success_status").show();
					jQuery("#flp_box_link").hide();
					jQuery("#flp_box_success").show();
				}
			}
			function sms_error2() {
				jQuery("#sms_err").html("Error: Could not perform SMS verification.");
				jQuery("#sms_err").show();
			}
			if(sessionStorage.wp_login_sms_vrf == "YES"){
				jQuery("#sms_verified").val("YES");
				jQuery("#sms_otp").hide();
				jQuery("#flp_submit_otp").hide();
				jQuery("#flp_get_otp").hide();
				jQuery("#flp_resend_otp").hide();
				jQuery("#reset_pn").hide();
				jQuery("#sms_phone_info").hide();
				jQuery("#sms_err").hide();
				jQuery("#flp_sms_box").hide();
				jQuery("#flp_sms").hide();
				jQuery("#flp_sms_success_status").show();
				jQuery("#flp_box_link").hide();
				jQuery("#flp_box_success").show();
			}
		</script>
		';
	}

	/**
	 * Send flp-verify.php link to EDD customer email.
	 */
	public function edd_sms_verification_email($payment_id) {
		$edd_sms_email = (null !== get_option('fraudlabs_pro_sms_verification_edd_email_verify')) ? get_option('fraudlabs_pro_sms_verification_edd_email_verify') : '';
		if ($edd_sms_email !== '1') {
			$this->write_debug_log('SMS Verification when order flagged for Review is not enable in EDD.');
			return;
		}

		// Get FraudLabs Pro Status.
		$result = get_post_meta($payment_id, '_edd_fraudlabspro');
		if (!is_array($result[0]) && !is_object($result[0])) {
			$flp_result = json_decode($result[0]);
		} else {
			$flp_result = $result[0];
		}

		if (count($result) > 0) {
			if (is_array($result[0])) {
				$flp_status = $flp_result['fraudlabspro_status'];
			} else {
				$flp_status = $flp_result->fraudlabspro_status;
			}

			if ($flp_status == '') {
				$this->write_debug_log('FraudLabs Pro Status not found. SMS Verification via email will not be performed for Payment ID ' . $payment_id . '.');
				return;
			}
		} else {
			$this->write_debug_log('FraudLabs Pro for Easy Digital Downloads plugin not installed. SMS Verification via email will not be performed for Payment ID ' . $payment_id . '.');
			return;
		}

		// Get Billing Email.
		$bill_email = get_post_meta($payment_id, '_edd_payment_user_email');
		if (count($bill_email) > 0) {
			$email = $bill_email[0];
		} else {
			$this->write_debug_log('Billing email not found. SMS Verification via email will not be performed for Payment ID ' . $payment_id . '.');
			return;
		}

		// Perform SMS verification via email for FraudLabs Pro Review.
		if ($flp_status == 'REVIEW') {
			if (filter_var ($email, FILTER_VALIDATE_EMAIL)) {
				$code = $this->randomCode(20);
				$link = get_site_url() . '/flp-verify.php?orderid=' . $payment_id . '&code=' . $code . '&src=edd';
				add_post_meta($payment_id, '_fraudlabspro_sms_edd', array(
					'fraudlabspro_edd_email_code'		=> $code,
					'fraudlabspro_edd_email_phone'		=> '',
					'fraudlabspro_edd_email_sms'		=> '',
					'fraudlabspro_edd_email_source'		=> 'edd',
				));

				$email_title = (null !== get_option('fraudlabs_pro_sms_verification_edd_email_subject')) ? ((get_option('fraudlabs_pro_sms_verification_edd_email_subject') == '') ? "Action Required: SMS Verification is required to process the order." : get_option('fraudlabs_pro_sms_verification_edd_email_subject')) : "Action Required: SMS Verification is required to process the order.";
				$email_content = (null !== get_option('fraudlabs_pro_sms_verification_edd_email_content')) ? ((get_option('fraudlabs_pro_sms_verification_edd_email_content') == '') ? "Dear Customer,\nThanks for your business. Before we can process your order, you are required to complete the SMS verification by clicking the below link:\n{email_verification_link}\nThank you." : get_option('fraudlabs_pro_sms_verification_edd_email_content')) : "Dear Customer,\nThanks for your business. Before we can process your order, you are required to complete the SMS verification by clicking the below link:\n{email_verification_link}\nThank you.";

				if (strpos($email_content, '{email_verification_link}') !== false) {
					$message = str_replace('{email_verification_link}', $link, $email_content);
				}

				$email_result = wp_mail($email, $email_title, $message);

				if ($email_result) {
					$this->write_debug_log('Email sent successfully for Payment ID ' . $payment_id . ' with code: ' . $code . '.');
				} else {
					$this->write_debug_log('Email sent fail for Payment ID ' . $payment_id . ' with code: ' . $code . '.');
				}
			} else {
				$this->write_debug_log('Billing email is not a valid email. SMS Verification via email will not be performed for Payment ID ' . $payment_id . '.');
				return;
			}
		}
	}

	/**
	 * Shortcode details for flp-sms-verification-edd.
	 */
	public function flp_sms_form_edd() {
		$table_name = $this->create_sms_table();

		$flp_ajax_field = 'flp_ajax_' . substr(hash('sha256', uniqid()), 8, 24);

		$ip = $this->get_user_ip();

		$smsdata = $this->query_sms_table($table_name, $ip);

		if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
			// Reset counter.
			$this->replace_sms_table($table_name, $ip, $this->sms_retries, time(), $flp_ajax_field);
			$smsdata = $this->query_sms_table($table_name, $ip);
		}
		else {
			// Just update ajax field.
			$this->replace_sms_table($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field);
		}
		// Refresh data.
		$smsdata = $this->query_sms_table($table_name, $ip);

		$retries = $smsdata->counter;
		$flp_sms_form_edd = '<div id="flp_sms_box" style="border:3px solid silver; padding:10px;">';
		$flp_sms_form_edd .= '<div id="sms_err" style="background-color:#f8d7da;color:#7d5880;padding:10px;margin-bottom:20px;font-size:1em;display:none;"></div>';
		if (!empty($this->sms_instruction)) $flp_sms_form_edd .= '<span id="sms_info_display" style="font-size:1.1em;">' . nl2br($this->sms_instruction) . ' <br /><br /></span>';
		$flp_sms_form_edd .= '<span id="sms_status">' . (($retries < 0) ? 'Maximum number of retries to send verification SMS exceeded. Please try again later.' : '') . '</span>';
		if (($retries >= 0)) {
			$flp_sms_form_edd .= '<label for="phone_number" id="enter_phone_number" style="font-weight:bold margin-top:10px; margin-bottom:10px;">Please enter Phone Number (with country code)<br /><input type="text" name="phone_number" id="phone_number" value="" style="width:100%;" /></label>';
		}
		$flp_sms_form_edd .= '<br /><div id="sms_otp" style="margin-top: 20px; display: none;"><span style="font-weight:bold;">Please enter OTP (One Time Passcode):</span><br /><input type="text" name="sms_otp1" id="sms_otp1" value="" style="width:130px; background-color:#bbb;">&nbsp;&nbsp;-&nbsp;&nbsp;<input type="text" name="sms_otp2" id="sms_otp2" value="" placeholder="Enter 6-digits OTP"></div>';
		$flp_sms_form_edd .= '<div style="margin-top:20px;"><input type="button" name="flp_submit_otp" id="flp_submit_otp" value="Submit OTP" style="margin-right: 10px; display: none;">';
		$flp_sms_form_edd .= '<input type="button" name="flp_get_otp" id="flp_get_otp" value="Request OTP" style="margin-right: 10px;' . (($retries < 0) ? 'display: none;' : '') . '">';
		$flp_sms_form_edd .= '<input type="button" name="flp_resend_otp" id="flp_resend_otp" value="Resend OTP" style="margin-right: 10px; display: none;">';
		$flp_sms_form_edd .= '<input type="hidden" name="sms_verified" id="sms_verified" value="">';
		$flp_sms_form_edd .= '<input type="hidden" name="sms_tran_id" id="sms_tran_id" value="">';
		$flp_sms_form_edd .= '<input type="hidden" name="sms_phone_cc" id="sms_phone_cc" value="' . get_option('fraudlabs_pro_sms_verification_sms_tel_cc') . '">';
		$flp_sms_form_edd .= '<input type="hidden" name="sms_retries" id="sms_retries" value="' . $retries . '">';
		$sms_order_id = (isset($_GET['orderid'])) ? sanitize_text_field($_GET['orderid']) : '';
		$sms_code = (isset($_GET['code'])) ? sanitize_text_field($_GET['code']) : '';
		$flp_sms_form_edd .= '<input type="hidden" name="sms_order_id" id="sms_order_id" value="' . $sms_order_id . '">';
		$flp_sms_form_edd .= '<input type="hidden" name="sms_code" id="sms_code" value="' . $sms_code . '">';
		$flp_sms_form_edd .= '</div></div>';
		$flp_sms_form_edd .= '<div id="flp_sms_success_status" style="border: 1px solid silver; padding: 10px; background-color: #22B14C; color: white; display: none;"><span>Your phone number has been successfully verified.</span></div><br />';

		$msgOtpSuccess = (get_option('fraudlabs_pro_sms_verification_msg_otp_success') == '') ? 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.' : get_option('fraudlabs_pro_sms_verification_msg_otp_success');
		$msgOtpSuccess = explode("{phone}", $msgOtpSuccess);
		$msgOtpFail = (get_option('fraudlabs_pro_sms_verification_msg_otp_fail') == '') ? 'Error: Unable to send the SMS verification message to {phone}.' : get_option('fraudlabs_pro_sms_verification_msg_otp_fail');
		$msgOtpFail = explode("{phone}", $msgOtpFail);
		$msgInvalidPhone = (get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') == '') ? 'Please enter a valid phone number.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_phone');
		$msgInvalidOtp = (get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') == '') ? 'Error: Invalid OTP. Please enter the correct OTP.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_otp');
		$flp_sms_form_edd .= '
		<style>
			input[type=text]:disabled {
				background: #dddddd;
			}
		</style>
		<script src="//ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>
		<script src="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/js/intlTelInput.min.js"></script>
		<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/css/intlTelInput.min.css">
		<script language="Javascript">
			var phoneNum;
			jQuery(document).ready(function() {
				phoneNum = window.intlTelInput(document.querySelector("#phone_number"), {
					utilsScript: "https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/js/utils.min.js",
					separateDialCode: true,
					initialCountry: jQuery("#sms_phone_cc").val()
				});
			});
			function doOTP() {
				var data = {
					"action": "wp_sms_action_send",
					"tel": phoneNum.getNumber(),
					"tel_cc": phoneNum.getSelectedCountryData().iso2.toUpperCase(),
					"sms_order_id": jQuery("#sms_order_id").val(),
					"source": "edd"
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success,
					error: sms_error,
					dataType: "text"
				});
			}
			function checkOTP() {
				var data = {
					"action": "wp_sms_action_verify",
					"otp": jQuery("#sms_otp1").val() + "-" + jQuery("#sms_otp2").val(),
					"tran_id": jQuery("#sms_tran_id").val(),
					"sms_order_id": jQuery("#sms_order_id").val(),
					"sms_code": jQuery("#sms_code").val(),
					"source": "edd"
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success2,
					error: sms_error2,
					dataType: "text"
				});
			}
			jQuery("#sms_otp2").bind("keypress", function(e) {
				var code = e.keyCode || e.which;
				if (code == 13) { //Enter keycode
					e.preventDefault();
				}
			});
			jQuery("#flp_get_otp").click(function(e) {
				if (jQuery("#phone_number").val() == "") {
					jQuery("#sms_err").html("'; $flp_sms_form_edd .= $msgInvalidPhone; $flp_sms_form_edd .= '");
					jQuery("#sms_err").show();
					jQuery("#phone_number").focus();
				} else if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_resend_otp").click(function(e) {
				if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_submit_otp").click(function() {
				checkOTP();
			});
			function sms_success(data) {
				// need to count down the number of retries
				jQuery("#sms_retries").val(jQuery("#sms_retries").val() - 1);
				if (data.indexOf("ERROR") == 0) {
					jQuery("#sms_err").html("'; $flp_sms_form_edd .= $msgOtpFail[0]; $flp_sms_form_edd .='" + phoneNum.getNumber() + "'; $flp_sms_form_edd .= $msgOtpFail[1]; $flp_sms_form_edd .='");
					jQuery("#sms_err").show();
					if (jQuery("#sms_retries").val() == 0) {
						jQuery("#enter_phone_number").hide();
						jQuery("#sms_otp").hide();
						jQuery("#flp_submit_otp").hide();
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
						jQuery("#sms_status").text("Maximum number of retries to send verification SMS exceeded. Please try again later.");
					}
				} else if (data.indexOf("OK") == 0) {
					alert("'; $flp_sms_form_edd .= $msgOtpSuccess[0]; $flp_sms_form_edd .='" + phoneNum.getNumber() + "'; $flp_sms_form_edd .= $msgOtpSuccess[1]; $flp_sms_form_edd .='");
					// store the tran_id
					jQuery("#sms_tran_id").val(data.substr(2,20));
					// show the submit otp button
					jQuery("#flp_get_otp").hide();
					jQuery("#sms_err").hide();
					jQuery("#flp_resend_otp").show();
					jQuery("#flp_submit_otp").show();
					jQuery("#sms_otp").show();
					jQuery("#sms_otp1").val(data.substr(22,6));
					jQuery("#phone_number").prop("disabled", true);
					jQuery("#sms_otp1").prop("disabled", true);
					if (jQuery("#sms_retries").val() == 0) {
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
					}
				}
			}
			function sms_error() {
				jQuery("#sms_err").html("'; $flp_sms_form_edd .= $msgOtpFail[0]; $flp_sms_form_edd .='" + phoneNum.getNumber() + "'; $flp_sms_form_edd .= $msgOtpFail[1]; $flp_sms_form_edd .='");
				jQuery("#sms_err").show();
			}
			function sms_success2(data) {
				if (data.indexOf("ERROR 601") == 0) {
					jQuery("#sms_err").html("'; $flp_sms_form_edd .= $msgInvalidOtp; $flp_sms_form_edd .= '");
					jQuery("#sms_err").show();
				} else if (data.indexOf("ERROR 600") == 0) {
					jQuery("#sms_err").html("Error: Error while performing verification.");
					jQuery("#sms_err").show();
				} else if (data.indexOf("OK") == 0) {
					// update the sms verified field
					jQuery("#sms_verified").val("YES");
					// hide all buttons and text field for the SMS...
					jQuery("#sms_otp").hide();
					jQuery("#flp_submit_otp").hide();
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").hide();
					jQuery("#sms_err").hide();
					// show SMS verified message
					jQuery("#flp_sms_box").hide();
					jQuery("#flp_sms_success_status").show();
					// redirect the page to get phone number
					var url = window.location.href + "&phone=" + phoneNum.getNumber();
					window.location.href = url;
				}
			}
			function sms_error2() {
				jQuery("#sms_err").html("Error: Could not perform SMS verification.");
				jQuery("#sms_err").show();
			}
		</script>
		';
		return $flp_sms_form_edd;
	}

	/**
	 * Write to debug log to record details of process.
	 */
	public function write_debug_log($message) {
		if (!get_option('fraudlabs_pro_sms_verification_debug_log_enabled')) {
			return;
		}

		if (is_array($message) || is_object($message)) {
			file_put_contents(FRAUDLABS_PRO_SMS_VERIFICATION_ROOT . 'debug.log', gmdate('Y-m-d H:i:s') . "\t" . print_r($message, true) . "\n", FILE_APPEND);
		} else {
			file_put_contents(FRAUDLABS_PRO_SMS_VERIFICATION_ROOT . 'debug.log', gmdate('Y-m-d H:i:s') . "\t" . $message . "\n", FILE_APPEND);
		}
	}

	public function plugin_enqueues($hook) {
		wp_enqueue_style('fraudlabs_pro_sms_verification_admin_menu_styles', plugins_url('/assets/css/style.css', __FILE__), array());

		if ($hook == 'plugins.php') {
			// Add in required libraries for feedback modal.
			wp_enqueue_script('jquery-ui-dialog');
			wp_enqueue_style('wp-jquery-ui-dialog');

			wp_enqueue_script('fraudlabs_pro_sms_verification_admin_script', plugins_url('/assets/js/feedback.js', __FILE__), ['jquery'], null, true);
		}
	}

	public function submit_feedback() {
		$feedback = (isset($_POST['feedback'])) ? sanitize_text_field($_POST['feedback']) : '';
		$others = (isset($_POST['others'])) ? sanitize_text_field($_POST['others']) : '';

		$options = [
			1 => 'I no longer need the plugin',
			2 => 'I couldn\'t get the plugin to work',
			3 => 'The plugin doesn\'t meet my requirements',
			4 => 'Other concerns' . (($others) ? (' - ' . $others) : ''),
		];

		if (isset($options[$feedback])) {
			if (!class_exists('WP_Http')) {
				include_once ABSPATH . WPINC . '/class-http.php';
			}

			$request = new WP_Http();
			$response = $request->request('https://www.fraudlabspro.com/wp-plugin-feedback?' . http_build_query([
				'name'    => 'fraudlabs-pro-sms-verification',
				'message' => $options[$feedback],
			]), ['timeout' => 5]);
		}
	}

	public function admin_footer_text($footer_text) {
		$plugin_name = 'fraudlabs-pro-sms-verification';
		$current_screen = get_current_screen();

		if (($current_screen && strpos($current_screen->id, 'fraudlabspro-sms-verification') !== false)) {
			$footer_text .= sprintf(
				__('<br />Enjoyed %1$s? Please leave us a %2$s rating. A huge thanks in advance!', $plugin_name),
				'<strong>' . __('FraudLabs Pro SMS Verification', $plugin_name) . '</strong>',
				'<a href="https://wordpress.org/support/plugin/' . $plugin_name . '/reviews/?filter=5/#new-post" target="_blank">&#9733;&#9733;&#9733;&#9733;&#9733;</a>'
			);
		}

		if ($current_screen->id == 'plugins') {
			return $footer_text . '
			<div id="fraudlabs-pro-sms-verification-feedback-modal" class="hidden" style="max-width:800px">
				<span id="fraudlabs-pro-sms-verification-feedback-response"></span>
				<p>
					<strong>Would you mind sharing with us the reason to deactivate the plugin?</strong>
				</p>
				<p>
					<label>
						<input type="radio" name="fraudlabs-pro-sms-verification-feedback" value="1"> I no longer need the plugin
					</label>
				</p>
				<p>
					<label>
						<input type="radio" name="fraudlabs-pro-sms-verification-feedback" value="2"> I couldn\'t get the plugin to work
					</label>
				</p>
				<p>
					<label>
						<input type="radio" name="fraudlabs-pro-sms-verification-feedback" value="3"> The plugin doesn\'t meet my requirements
					</label>
				</p>
				<p>
					<label>
						<input type="radio" name="fraudlabs-pro-sms-verification-feedback" value="4"> Other concerns
						<br /><br />
						<textarea id="fraudlabs-pro-sms-verification-feedback-other" style="display:none;width:100%"></textarea>
					</label>
				</p>
				<p>
					<div style="float:left">
						<input type="button" id="fraudlabs-pro-sms-verification-submit-feedback-button" class="button button-danger" value="Submit & Deactivate" />
					</div>
					<div style="float:right">
						<a href="#">Skip & Deactivate</a>
					</div>
				</p>
			</div>';
		}

		return $footer_text;
	}

	/**
	 * AJAX function for sms sending.
	 */
	function wp_sms_action_send() {
		global $wpdb;
		$table_name = $wpdb->prefix . 'flp_sms_counter';
		$ip = $this->get_user_ip();
		$smsdata = $this->query_sms_table_tel($table_name, $ip, sanitize_text_field($_POST['tel']));
		if (($smsdata === NULL)) {
			$smsdata = $this->query_sms_table($table_name, $ip);
		}

		$sms_order_id = (isset($_POST['sms_order_id'])) ? sanitize_text_field($_POST['sms_order_id']) : "";
		if ($sms_order_id != "") {
			$result = get_post_meta($sms_order_id, '_edd_fraudlabspro');
			if (!is_array($result[0]) && !is_object($result[0])) {
				$row = json_decode($result[0]);
			} else {
				$row = $result[0];
			}

			if (is_array($result[0])) {
				$flpId = $row['fraudlabspro_id'];
			} else {
				$flpId = $row->fraudlabspro_id;
			}
		} else {
			$flpId = '';
		}

		$params['source'] = 'woocommerce';
		$params['tel'] = trim(sanitize_text_field($_POST['tel']));
		if (strpos($params['tel'], '+') !== 0)
			$params['tel'] = '+' . $params['tel'];
		$params['mesg'] = $this->sms_template;
		$params['mesg'] = str_replace(['{', '}'], ['<', '>'], $params['mesg']);
		$params['flp_id'] = $flpId;
		$params['tel_cc'] = (isset($_POST['tel_cc'])) ? sanitize_text_field($_POST['tel_cc']) : "";
		$params['otp_timeout'] = ($this->otp_timeout == '') ? '3600' : $this->otp_timeout;
		$url = 'https://api.fraudlabspro.com/v1/verification/send';

		$smsdata->counter = $smsdata->counter-1;
		$this->replace_sms_table_tel($table_name, $ip, $smsdata->counter, $smsdata->last, $smsdata->ajax, sanitize_text_field($_POST['tel']));

		$query = '';
		foreach( $params as $key=>$value ) {
			$query .= '&' . $key . '=' . rawurlencode( $value );
		}
		$url = $url . '?key=' . $this->api_key . $query;
		$request = wp_remote_get( $url );

		// Network error, wait 2 seconds for next retry
		if (is_wp_error($request)) {
			$this->write_debug_log( 'Network error during FraudLabs Pro SMS OTP Send. Retrying.' );
			for ($i = 0; $i < 3; ++$i ) {
				sleep(2);
				$request = wp_remote_get( $url );
				
				if (!is_wp_error($request)) {
					break;
				}
			}
		}

		// Still having network issue after 3 retries
		if (is_wp_error($request)) {
			$this->write_debug_log('Network error during FraudLabs Pro SMS OTP Send. SMS Verification aborted.');
			die ('ERROR 500');
		}

		// Get the HTTP response
		$data = json_decode(wp_remote_retrieve_body($request));

		if (trim($data->error) != '') {
			$this->write_debug_log('Error occurred during FraudLabs Pro SMS OTP Send to ' . sanitize_text_field($_POST['tel']) . '. ERROR: ' . $data->error);
			die('ERROR 600 ' . $smsdata->counter);
		} else {
			die ('OK' . $data->tran_id . $data->otp_char. $smsdata->counter);
		}
	}

	/**
	 * AJAX function for SMS verifying.
	 */
	function wp_sms_action_verify() {
		global $wpdb;
		$table_name = $wpdb->prefix . 'flp_sms_counter';
		$ip = $this->get_user_ip();

		$params['otp'] = (isset($_POST['otp'])) ? sanitize_text_field($_POST['otp']) : die('ERROR 400');
		$params['tran_id'] = (isset($_POST['tran_id'])) ? sanitize_text_field($_POST['tran_id']) : die('ERROR 400');
		$source = isset($_POST['source']) ? sanitize_text_field($_POST['source']) : '';

		require_once FRAUDLABS_PRO_SMS_VERIFICATION_ROOT . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

		// Configures FraudLabs Pro API key.
		$config = new FraudLabsPro\Configuration($this->api_key);
		$fraudlabsproSms = new \FraudLabsPro\SmsVerification($config);

		$data = $fraudlabsproSms->verifyOtp([
			'tran_id'		=> $params['tran_id'],
			'otp'			=> $params['otp'],
		]);

		if (trim($data->error) != '') {
			if ($data->error == 'Invalid OTP.') {
				die('ERROR 601-' . $data->error);
			} else {
				$this->write_debug_log('Error occurred during FraudLabs Pro SMS OTP Verify. ERROR: ' . $data->error);
				die('ERROR 600-' . $data->error);
			}
		}
		else {
			$this->delete_sms_table($table_name, $ip);

			if ($source == "edd") {
				$sms_order_id = (isset($_POST['sms_order_id'])) ? sanitize_text_field($_POST['sms_order_id']) : "";
				$sms_code = (isset($_POST['sms_code'])) ? sanitize_text_field($_POST['sms_code']) : "";
				if ($sms_order_id != "") {
					if ($sms_code != "") {
						$result = get_post_meta($sms_order_id, '_fraudlabspro_sms_edd');
						if (!is_array($result[0]) && !is_object($result[0])) {
							$sms_verify = json_decode($result[0]);
						} else {
							$sms_verify = $result[0];
						}

						if (is_array($result[0])) {
							if ($sms_verify['fraudlabspro_edd_email_code'] == $sms_code) {
								$sms_verify['fraudlabspro_edd_email_code'] = $sms_code . '_VERIFIED';
								update_post_meta($sms_order_id, '_fraudlabspro_sms_edd', $sms_verify);
							}
						} else {
							if ($sms_verify->fraudlabspro_edd_email_code == $sms_code) {
								$sms_verify->fraudlabspro_edd_email_code = $sms_code . '_VERIFIED';
								update_post_meta($sms_order_id, '_fraudlabspro_sms_edd', $sms_verify);
							}
						}

					}
				}
			}

			die ('OK');
		}
	}

	/**
	 * Delete data in SMS table.
	 */
	private function delete_sms_table($table_name, $ip) {
		global $wpdb;
		
		$wpdb->delete($table_name, array('ip' => $ip), array('%s'));
	}

	/**
	 * Create SMS counter table.
	 */
	private function create_sms_table() {
		global $wpdb;
		$charset_collate = $wpdb->get_charset_collate();
		$table_name = $wpdb->prefix . 'flp_sms_counter';

		$sql = "CREATE TABLE IF NOT EXISTS $table_name (
			`ip` varchar(50) NOT NULL,
			`counter` int DEFAULT '0',
			`last` int DEFAULT '0',
			`ajax` varchar(50) DEFAULT '',
			`phone` varchar(20) DEFAULT '0',
			PRIMARY KEY (`ip`, `phone`)
		) $charset_collate;";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql); // This will check if table structure needs to be updated and do it automatically.

		return $table_name;
	}

	/**
	 * Gets user IP.
	 */
	private function get_user_ip() {
		if (isset($_SERVER['HTTP_X_FORWARDED_FOR']) && $this->isvalidip($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			$ip_address = $_SERVER['HTTP_X_FORWARDED_FOR'];
		}
		else {
			$ip_address = $_SERVER['REMOTE_ADDR'];
		}

		if (preg_match('/,/', $ip_address)) {
			$tmp = explode(',', $ip_address);
			$ip_address = trim($tmp[0]);
		}
		return $ip_address;
	}

	/**
	 * Check for valid IP.
	 */
	private function isvalidip($ip) {
		if ((filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) || (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6))) {
			return true;
		}
		else {
			return false;
		}
	}

	/**
	 * Query SMS counter table.
	 */
	private function query_sms_table($table_name, $ip) {
		global $wpdb;

		return $wpdb->get_row("SELECT * FROM $table_name WHERE ip = '" . $ip . "' LIMIT 1");
	}
	private function query_sms_table_tel($table_name, $ip, $phone) {
		global $wpdb;

		return $wpdb->get_row("SELECT * FROM $table_name WHERE ip = '" . $ip . "' AND phone = '" . $phone . "' LIMIT 1");
	}

	/**
	 * Update SMS counter table.
	 */
	private function replace_sms_table($table_name, $ip, $counter, $last, $ajax) {
		global $wpdb;

		$wpdb->replace($table_name, array('ip' => $ip, 'counter' => $counter, 'last' => $last, 'ajax' => $ajax), array('%s', '%d', '%d', '%s'));
	}
	private function replace_sms_table_tel($table_name, $ip, $counter, $last, $ajax, $phone) {
		global $wpdb;

		$wpdb->replace($table_name, array('ip' => $ip, 'counter' => $counter, 'last' => $last, 'ajax' => $ajax, 'phone' => $phone), array('%s', '%d', '%d', '%s', '%s'));
	}

	/**
	 * Generate random code for email verification.
	 */
	private function randomCode($length=16) {
		$key = '';
		$pattern = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		for($i=0; $i<$length; $i++) {
			$key .= $pattern[rand(0, strlen($pattern)-1)];
		}
		return $key;
	}

	/**
	 * Check if the param exists in the array.
	 */
	private function is_in_array($needle, $array) {
		foreach (array_values($array) as $key) {
			$return[$key] = 1;
		}

		return isset($return[$needle]);
	}

	/**
	 * Create admin tab.
	 */
	private function admin_tabs()
	{
		$disable_tabs = false;

		$tab = (isset($_GET['tab'])) ? sanitize_text_field($_GET['tab']) : 'settings';

		return '
		<h2 class="nav-tab-wrapper">
			<a href="' . (($disable_tabs) ? 'javascript:;' : admin_url('admin.php?page=fraudlabspro-sms-verification&tab=settings')) . '" class="nav-tab' . (($tab == 'settings') ? ' nav-tab-active' : '') . '">General Settings</a>
			<a href="' . (($disable_tabs) ? 'javascript:;' : admin_url('admin.php?page=fraudlabspro-sms-verification&tab=wordpress')) . '" class="nav-tab' . (($tab == 'wordpress') ? ' nav-tab-active' : '') . '">WordPress</a>
			<a href="' . (($disable_tabs) ? 'javascript:;' : admin_url('admin.php?page=fraudlabspro-sms-verification&tab=woocommerce')) . '" class="nav-tab' . (($tab == 'woocommerce') ? ' nav-tab-active' : '') . '">WooCommerce</a>
			<a href="' . (($disable_tabs) ? 'javascript:;' : admin_url('admin.php?page=fraudlabspro-sms-verification&tab=edd')) . '" class="nav-tab' . (($tab == 'edd') ? ' nav-tab-active' : '') . '">Easy Digital Downloads</a>
			<a href="' . (($disable_tabs) ? 'javascript:;' : admin_url('admin.php?page=fraudlabspro-sms-verification&tab=shortcodes')) . '" class="nav-tab' . (($tab == 'shortcodes') ? ' nav-tab-active' : '') . '">Shortcodes</a>
		</h2>';
	}
}

$fraudlabspro_sms_verification = new FraudLabsPro_SMS_Verification(__FILE__);
register_activation_hook(__FILE__, array($fraudlabspro_sms_verification, 'set_defaults'));

endif;
