<?php
/**
 * FraudLabs Pro SMS Verification Integration.
 *
 * @package  WC_Integration_FraudLabs_Pro_SMS_Verification
 * @category Integration
 * @author   FraudLabs Pro
 */

if (!class_exists('WC_Integration_FraudLabs_Pro_SMS_Verification')) :

class WC_Integration_FraudLabs_Pro_SMS_Verification extends WC_Integration {

	/**
	 * Initializes and hook in the integration.
	 */
	public function __construct() {
		$this->namespace			= 'woocommerce-fraudlabs-pro-sms-verification';
		$this->method_title			= __('FraudLabs Pro SMS Verification', 'woocommerce-fraudlabs-pro-sms-verification');
		$this->method_description	= __('FraudLabs Pro SMS verification help merchants to authenticate the client\'s identity by sending them a SMS for verification. Get a <a href="https://www.fraudlabspro.com/sign-up?r=woocommerce" target="_blank">free API key</a> if you do not have one.', 'woocommerce-fraudlabs-pro-sms-verification');

		// Load the settings.
		$this->init_settings();

		// Hooks for WooCommerce.
		add_action('manage_shop_order_posts_custom_column', array($this, 'render_column'), 3);
		add_action('woocommerce_checkout_after_terms_and_conditions', array($this, 'woocommerce_checkout_form'));
		add_action('woocommerce_checkout_process', array($this, 'woocommerce_checkout_process_sms'));
		add_action('woocommerce_register_form', array($this, 'woocommerce_register_form'));
		add_action('woocommerce_thankyou', array($this, 'woocommerce_thankyou_email'));
		add_filter('manage_shop_order_posts_columns', array($this, 'add_column'), 11);

		add_action('admin_enqueue_scripts', array($this, 'plugin_enqueues'));
		add_action('admin_notices', array($this, 'admin_notifications'));
		add_action('wp_ajax_fraudlabspro_sms_verification_wc_admin_notice', array($this, 'plugin_dismiss_admin_notice'));
		add_action('wp_ajax_wc_sms_action_send', array($this, 'wc_sms_action_send'));
		add_action('wp_ajax_wc_sms_action_verify', array($this, 'wc_sms_action_verify'));

		// Shortcode for SMS verification via email.
		add_shortcode('flp-sms-verification-wc', array($this, 'flp_sms_form_wc'));

		if (isset($_REQUEST['action']) && $_REQUEST['action']=='wc_sms_action_send') {
			add_action('wp_ajax_wc_sms_action_send', array($this, 'wc_sms_action_send'));
			do_action('wp_ajax_' . $_REQUEST['action']);
		}
		if (isset($_REQUEST['action']) && $_REQUEST['action']=='wc_sms_action_verify') {
			add_action('wp_ajax_wc_sms_action_verify', array($this, 'wc_sms_action_verify'));
			do_action('wp_ajax_' . $_REQUEST['action']);
		}
	}

	/**
	 * Add SMS verified column to order list.
	 */
	public function add_column($columns) {
		if (get_option('fraudlabs_pro_sms_verification_wc_email_verify') == 0) {
			return $columns;
		}

		$columns = array_merge(array_slice($columns, 0, 6), array('fraudlabspro_sms_verified' => 'SMS Verified'), array_slice($columns, 6));
		return $columns;
	}

	/**
	 * Fill in SMS verified info into SMS verified column.
	 */
	public function render_column($column) {
		if (get_option('fraudlabs_pro_sms_verification_wc_email_verify') == 0) {
			return;
		}

		if ($column != 'fraudlabspro_sms_verified') {
			return;
		}

		global $post;

		$result = get_post_meta($post->ID, '_fraudlabspro');

		if (count($result) > 0) {
			if (!is_array($result[0]) && !is_object($result[0])) {
				$row = json_decode($result[0]);
			} else {
				$row = $result[0];
			}

			if (is_array($result[0])) {
				if (is_null($row) === FALSE) {
					if (isset($row['is_phone_verified'])) {
						if ($row['is_phone_verified'] == 'No') {
							echo '<div style="color:#ff0000"><strong>No</strong></div>';
						}
						else {
							echo '<div style="color:#66cc00"><strong>Yes</strong></div>';
						}
					}
					else {
						echo '<div style="color:#f0c850">NA</div>';
					}
				}
			} else {
				if (is_null($row) === FALSE) {
					if (isset($row->is_phone_verified)) {
						if ($row->is_phone_verified == 'No') {
							echo '<div style="color:#ff0000"><strong>No</strong></div>';
						}
						else {
							echo '<div style="color:#66cc00"><strong>Yes</strong></div>';
						}
					}
					else {
						echo '<div style="color:#f0c850">NA</div>';
					}
				}
			}
		}
	}

	/**
	 * Send flp-verify.php link to customer email.
	 */
	public function woocommerce_thankyou_email($order_id) {
		if (get_option('fraudlabs_pro_sms_verification_wc_email_verify') == 0) {
			return;
		}

		// Get FraudLabs Pro Status.
		$result = get_post_meta($order_id, '_fraudlabspro');
		if (!is_array($result[0]) && !is_object($result[0])) {
			$flp_result = json_decode($result[0]);
		} else {
			$flp_result = $result[0];
		}

		if (count($result) > 0) {
			if (is_array($result[0])) {
				$flp_status = $flp_result['fraudlabspro_status'];
			} else {
				$flp_status = $flp_result->fraudlabspro_status;
			}

			if ($flp_status == '') {
				$this->write_debug_log('FraudLabs Pro Status not found. Email Verification will not be performed for Order ID ' . $order_id . '.');
				return;
			}
		} else {
			$this->write_debug_log('FraudLabs Pro for WooCommerce plugin not installed. Email Verification will not be performed for Order ID ' . $order_id . '.');
			return;
		}

		// Get Billing Email.
		$bill_email = get_post_meta($order_id, '_billing_email');
		if (count($bill_email) > 0) {
			$email = $bill_email[0];
		} else {
			$this->write_debug_log('Billing email not found. Email Verification will not be performed for Order ID ' . $order_id . '.');
			return;
		}

		// Perform email verification for FraudLabs Pro Review.
		if ($flp_status == 'REVIEW') {
			if (filter_var ($email, FILTER_VALIDATE_EMAIL)) {
				$code = $this->randomCode(20);
				$link = get_site_url() . '/flp-verify.php?orderid=' . $order_id . '&code=' . $code;
				add_post_meta($order_id, '_fraudlabspro_sms_wc', array(
					'fraudlabspro_wc_email_code'		=> $code,
					'fraudlabspro_wc_email_phone'		=> '',
					'fraudlabspro_wc_email_sms'			=> '',
				));

				$email_title = (null !== get_option('fraudlabs_pro_sms_verification_wc_email_subject')) ? ((get_option('fraudlabs_pro_sms_verification_wc_email_subject') == '') ? "Action Required: SMS Verification is required to process the order." : get_option('fraudlabs_pro_sms_verification_wc_email_subject')) : "Action Required: SMS Verification is required to process the order.";
				$email_content = (null !== get_option('fraudlabs_pro_sms_verification_wc_email_content')) ? ((get_option('fraudlabs_pro_sms_verification_wc_email_content') == '') ? "Dear Customer,\nThanks for your business. Before we can process your order, you are required to complete the SMS verification by clicking the below link:\n{email_verification_link}\nThank you." : get_option('fraudlabs_pro_sms_verification_wc_email_content')) : "Dear Customer,\nThanks for your business. Before we can process your order, you are required to complete the SMS verification by clicking the below link:\n{email_verification_link}\nThank you.";

				if (strpos($email_content, '{email_verification_link}') !== false) {
					$message = str_replace('{email_verification_link}', $link, $email_content);
				}

				$email_result = wp_mail($email, $email_title, $message);

				if ($email_result) {
					$this->write_debug_log('Email sent successfully for Order ID ' . $order_id . ' with code: ' . $code . '.');
				} else {
					$this->write_debug_log('Email sent fail for Order ID ' . $order_id . ' with code: ' . $code . '.');
				}
			} else {
				$this->write_debug_log('Billing email is not a valid email. Email Verification will not be performed for Order ID ' . $order_id . '.');
				return;
			}
		}
	}

	/**
	 * Shortcode details for flp-sms-verification-wc.
	 */
	public function flp_sms_form_wc() {
		$table_name = $this->create_sms_table();

		$flp_ajax_field = 'flp_ajax_' . substr(hash('sha256', uniqid()), 8, 24);

		$ip = $this->get_user_ip();

		$smsdata = $this->query_sms_table($table_name, $ip);

		if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
			// Reset counter.
			$this->replace_sms_table($table_name, $ip, get_option('fraudlabs_pro_sms_verification_sms_retries'), time(), $flp_ajax_field);
			$smsdata = $this->query_sms_table($table_name, $ip);
		}
		else {
			// Just update ajax field.
			$this->replace_sms_table($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field);
		}
		// Refresh data.
		$smsdata = $this->query_sms_table($table_name, $ip);

		$retries = $smsdata->counter;
		$flp_sms_form_wc = '<div id="flp_sms_box" style="border:3px solid silver; padding:10px;">';
		$flp_sms_form_wc .= '<div id="sms_err" style="background-color:#f8d7da;color:#7d5880;padding:10px;margin-bottom:20px;font-size:1em;display:none;"></div>';
		if (!empty(get_option('fraudlabs_pro_sms_verification_sms_instruction'))) $flp_sms_form_wc .= '<span id="sms_info_display" style="font-size:1.1em;">' . nl2br(get_option('fraudlabs_pro_sms_verification_sms_instruction')) . ' <br /><br /></span>';
		$flp_sms_form_wc .= '<span id="sms_status">' . (($retries < 0) ? 'Maximum number of retries to send verification SMS exceeded. Please try again later.' : '') . '</span>';
		if (($retries >= 0)) {
			$flp_sms_form_wc .= '<label for="phone_number" id="enter_phone_number" style="font-weight:bold">Please enter Phone Number (with country code)<br /><input type="text" name="phone_number" id="phone_number" value="" style="width:100%;" /></label>';
		}
		$flp_sms_form_wc .= '<br /><div id="sms_otp" style="margin-top: 20px; display: none;"><span style="font-weight:bold;">Please enter OTP (One Time Passcode):</span><br/><input type="text" name="sms_otp1" id="sms_otp1" value="" style="width:130px; background-color:#bbb;">&nbsp;&nbsp;-&nbsp;&nbsp;<input type="text" name="sms_otp2" id="sms_otp2" value="" placeholder="Enter 6-digits OTP"></div>';
		$flp_sms_form_wc .= '<div style="margin-top:20px;"><input type="button" name="flp_submit_otp" id="flp_submit_otp" value="Submit OTP" style="margin-right: 10px; display: none;">';
		$flp_sms_form_wc .= '<input type="button" name="flp_get_otp" id="flp_get_otp" value="Request OTP" style="margin-right: 10px;' . (($retries < 0) ? 'display: none;' : '') . '">';
		$flp_sms_form_wc .= '<input type="button" name="flp_resend_otp" id="flp_resend_otp" value="Resend OTP" style="margin-right: 10px; display: none;">';
		$flp_sms_form_wc .= '<input type="hidden" name="sms_verified" id="sms_verified" value="">';
		$flp_sms_form_wc .= '<input type="hidden" name="sms_tran_id" id="sms_tran_id" value="">';
		$flp_sms_form_wc .= '<input type="hidden" name="sms_phone_cc" id="sms_phone_cc" value="' . get_option('fraudlabs_pro_sms_verification_sms_tel_cc') . '">';
		$flp_sms_form_wc .= '<input type="hidden" name="sms_retries" id="sms_retries" value="' . $retries . '">';
		$sms_order_id = (isset($_GET['orderid'])) ? sanitize_text_field($_GET['orderid']) : '';
		$sms_code = (isset($_GET['code'])) ? sanitize_text_field($_GET['code']) : '';
		$flp_sms_form_wc .= '<input type="hidden" name="sms_order_id" id="sms_order_id" value="' . $sms_order_id . '">';
		$flp_sms_form_wc .= '<input type="hidden" name="sms_code" id="sms_code" value="' . $sms_code . '">';
		$flp_sms_form_wc .= '</div></div>';
		$flp_sms_form_wc .= '<div id="flp_sms_success_status" style="border: 1px solid silver; padding: 10px; background-color: #22B14C; color: white; display: none;"><span>Your phone number has been successfully verified.</span></div><br />';

		$msgOtpSuccess = (get_option('fraudlabs_pro_sms_verification_msg_otp_success') == '') ? 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.' : get_option('fraudlabs_pro_sms_verification_msg_otp_success');
		$msgOtpSuccess = explode("{phone}", $msgOtpSuccess);
		$msgOtpFail = (get_option('fraudlabs_pro_sms_verification_msg_otp_fail') == '') ? 'Error: Unable to send the SMS verification message to {phone}.' : get_option('fraudlabs_pro_sms_verification_msg_otp_fail');
		$msgOtpFail = explode("{phone}", $msgOtpFail);
		$msgInvalidPhone = (get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') == '') ? 'Please enter a valid phone number.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_phone');
		$msgInvalidOtp = (get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') == '') ? 'Error: Invalid OTP. Please enter the correct OTP.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_otp');
		$flp_sms_form_wc .= '
		<style>
			input[type=text]:disabled {
				background: #dddddd;
			}
		</style>
		<script src="//ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>
		<script src="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/js/intlTelInput.min.js"></script>
		<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/css/intlTelInput.min.css">
		<script language="Javascript">
			var phoneNum;
			jQuery(document).ready(function() {
				phoneNum = window.intlTelInput(document.querySelector("#phone_number"), {
					utilsScript: "https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.5/js/utils.min.js",
					separateDialCode: true,
					initialCountry: jQuery("#sms_phone_cc").val()
				});
			});
			function doOTP() {
				var data = {
					"action": "wc_sms_action_send",
					"tel": phoneNum.getNumber(),
					"tel_cc": phoneNum.getSelectedCountryData().iso2.toUpperCase(),
					"sms_order_id": jQuery("#sms_order_id").val()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success,
					error: sms_error,
					dataType: "text"
				});
			}
			function checkOTP() {
				var data = {
					"action": "wc_sms_action_verify",
					"otp": jQuery("#sms_otp1").val() + "-" + jQuery("#sms_otp2").val(),
					"tran_id": jQuery("#sms_tran_id").val(),
					"sms_order_id": jQuery("#sms_order_id").val(),
					"sms_code": jQuery("#sms_code").val(),
					"source": "wc_shortcode"
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success2,
					error: sms_error2,
					dataType: "text"
				});
			}
			jQuery("#sms_otp2").bind("keypress", function(e) {
				var code = e.keyCode || e.which;
				if (code == 13) { //Enter keycode
					e.preventDefault();
				}
			});
			jQuery("#flp_get_otp").click(function(e) {
				if (jQuery("#phone_number").val() == "") {
					jQuery("#sms_err").html("'; $flp_sms_form_wc .= $msgInvalidPhone; $flp_sms_form_wc .= '");
					jQuery("#sms_err").show();
					jQuery("#phone_number").focus();
				} else if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_resend_otp").click(function(e) {
				if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_submit_otp").click(function() {
				checkOTP();
			});
			function sms_success(data) {
				// need to count down the number of retries
				jQuery("#sms_retries").val(jQuery("#sms_retries").val() - 1);
				if (data.indexOf("ERROR") == 0) {
					jQuery("#sms_err").html("'; $flp_sms_form_wc .= $msgOtpFail[0]; $flp_sms_form_wc .='" + phoneNum.getNumber() + "'; $flp_sms_form_wc .= $msgOtpFail[1]; $flp_sms_form_wc .='");
					jQuery("#sms_err").show();
					if (jQuery("#sms_retries").val() == 0) {
						jQuery("#enter_phone_number").hide();
						jQuery("#sms_otp").hide();
						jQuery("#flp_submit_otp").hide();
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
						jQuery("#sms_status").text("Maximum number of retries to send verification SMS exceeded. Please try again later.");
					}
				} else if (data.indexOf("OK") == 0) {
					alert("'; $flp_sms_form_wc .= $msgOtpSuccess[0]; $flp_sms_form_wc .='" + phoneNum.getNumber() + "'; $flp_sms_form_wc .= $msgOtpSuccess[1]; $flp_sms_form_wc .='");
					// store the tran_id
					jQuery("#sms_tran_id").val(data.substr(2,20));
					// show the submit otp button
					jQuery("#flp_get_otp").hide();
					jQuery("#sms_err").hide();
					jQuery("#flp_resend_otp").show();
					jQuery("#flp_submit_otp").show();
					jQuery("#sms_otp").show();
					jQuery("#sms_otp1").val(data.substr(22,6));
					jQuery("#phone_number").prop("disabled", true);
					jQuery("#sms_otp1").prop("disabled", true);
					if (jQuery("#sms_retries").val() == 0) {
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
					}
				}
			}
			function sms_error() {
				jQuery("#sms_err").html("'; $flp_sms_form_wc .= $msgOtpFail[0]; $flp_sms_form_wc .='" + phoneNum.getNumber() + "'; $flp_sms_form_wc .= $msgOtpFail[1]; $flp_sms_form_wc .='");
				jQuery("#sms_err").show();
			}
			function sms_success2(data) {
				if (data.indexOf("ERROR 601") == 0) {
					jQuery("#sms_err").html("'; $flp_sms_form_wc .= $msgInvalidOtp; $flp_sms_form_wc .= '");
					jQuery("#sms_err").show();
				} else if (data.indexOf("ERROR 600") == 0) {
					jQuery("#sms_err").html("Error: Error while performing verification.");
					jQuery("#sms_err").show();
				} else if (data.indexOf("OK") == 0) {
					// update the sms verified field
					jQuery("#sms_verified").val("YES");
					// hide all buttons and text field for the SMS...
					jQuery("#sms_otp").hide();
					jQuery("#flp_submit_otp").hide();
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").hide();
					jQuery("#sms_err").hide();
					// show SMS verified message
					jQuery("#flp_sms_box").hide();
					jQuery("#flp_sms_success_status").show();
					// redirect the page to get phone number
					var url = window.location.href + "&phone=" + phoneNum.getNumber();
					window.location.href = url;
				}
			}
			function sms_error2() {
				jQuery("#sms_err").html("Error: Could not perform SMS verification.");
				jQuery("#sms_err").show();
			}
		</script>
		';
		return $flp_sms_form_wc;
	}

	/**
	 * Process the WC Checkout form to check if SMS verification is required.
	 */
	public function woocommerce_checkout_process_sms() {
		if (get_option('fraudlabs_pro_sms_verification_wc_default_checkout_form') == 0) {
			$this->write_debug_log('SMS Verification on WooCommerce Checkout Form not enabled. SMS Verification will not be performed.');
			return;
		}
		elseif ((trim(get_option('fraudlabs_pro_sms_verification_api_key')) == '')) {
			$this->write_debug_log('FraudLabs Pro API Key not inserted. SMS Verification will not be performed.');
			return;
		}

		if ($_POST['sms_verified']) {
			return;
		}

		$idd = ['AD' => '376', 'AE' => '971', 'AF' => '93', 'AG' => '1268', 'AI' => '1264', 'AL' => '355', 'AM' => '374', 'AO' => '244', 'AQ' => '672', 'AR' => '54', 'AS' => '1684', 'AT' => '43', 'AU' => '61', 'AW' => '297', 'AX' => '358', 'AZ' => '994', 'BA' => '387', 'BB' => '1246', 'BD' => '880', 'BE' => '32', 'BF' => '226', 'BG' => '359', 'BH' => '973', 'BI' => '257', 'BJ' => '229', 'BL' => '590', 'BM' => '1441', 'BN' => '673', 'BO' => '591', 'BQ' => '599', 'BR' => '55', 'BS' => '1242', 'BT' => '975', 'BV' => '47', 'BW' => '267', 'BY' => '375', 'BZ' => '501', 'CA' => '1', 'CC' => '61', 'CD' => '243', 'CF' => '236', 'CG' => '242', 'CH' => '41', 'CI' => '225', 'CK' => '682', 'CL' => '56', 'CM' => '237', 'CN' => '86', 'CO' => '57', 'CR' => '506', 'CU' => '53', 'CV' => '238', 'CW' => '599', 'CX' => '61', 'CY' => '357', 'CZ' => '420', 'DE' => '49', 'DJ' => '253', 'DK' => '45', 'DM' => '1767', 'DO' => '1809', 'DZ' => '213', 'EC' => '593', 'EE' => '372', 'EG' => '20', 'EH' => '212', 'ER' => '291', 'ES' => '34', 'ET' => '251', 'FI' => '358', 'FJ' => '679', 'FK' => '500', 'FM' => '691', 'FO' => '298', 'FR' => '33', 'GA' => '241', 'GB' => '44', 'GD' => '1473', 'GE' => '995', 'GF' => '594', 'GG' => '44', 'GH' => '233', 'GI' => '350', 'GL' => '299', 'GM' => '220', 'GN' => '224', 'GP' => '590', 'GQ' => '240', 'GR' => '30', 'GS' => '500', 'GT' => '502', 'GU' => '1671', 'GW' => '245', 'GY' => '592', 'HK' => '852', 'HM' => '61', 'HN' => '504', 'HR' => '385', 'HT' => '509', 'HU' => '36', 'ID' => '62', 'IE' => '353', 'IL' => '972', 'IM' => '44', 'IN' => '91', 'IO' => '246', 'IQ' => '964', 'IR' => '98', 'IS' => '354', 'IT' => '39', 'JE' => '44', 'JM' => '1876', 'JO' => '962', 'JP' => '81', 'KE' => '254', 'KG' => '996', 'KH' => '855', 'KI' => '686', 'KM' => '269', 'KN' => '1869', 'KP' => '850', 'KR' => '82', 'KW' => '965', 'KY' => '1345', 'KZ' => '7', 'LA' => '856', 'LB' => '961', 'LC' => '1758', 'LI' => '423', 'LK' => '94', 'LR' => '231', 'LS' => '266', 'LT' => '370', 'LU' => '352', 'LV' => '371', 'LY' => '218', 'MA' => '212', 'MC' => '377', 'MD' => '373', 'ME' => '382', 'MF' => '590', 'MG' => '261', 'MH' => '692', 'MK' => '389', 'ML' => '223', 'MM' => '95', 'MN' => '976', 'MO' => '853', 'MP' => '1670', 'MQ' => '596', 'MR' => '222', 'MS' => '1664', 'MT' => '356', 'MU' => '230', 'MV' => '960', 'MW' => '265', 'MX' => '52', 'MY' => '60', 'MZ' => '258', 'NA' => '264', 'NC' => '687', 'NE' => '227', 'NF' => '672', 'NG' => '234', 'NI' => '505', 'NL' => '31', 'NO' => '47', 'NP' => '977', 'NR' => '674', 'NU' => '683', 'NZ' => '64', 'OM' => '968', 'PA' => '507', 'PE' => '51', 'PF' => '689', 'PG' => '675', 'PH' => '63', 'PK' => '92', 'PL' => '48', 'PM' => '508', 'PN' => '64', 'PR' => '1787', 'PS' => '970', 'PT' => '351', 'PW' => '680', 'PY' => '595', 'QA' => '974', 'RE' => '262', 'RO' => '40', 'RS' => '381', 'RU' => '7', 'RW' => '250', 'SA' => '966', 'SB' => '677', 'SC' => '248', 'SD' => '249', 'SE' => '46', 'SG' => '65', 'SH' => '290', 'SI' => '386', 'SJ' => '47', 'SK' => '421', 'SL' => '232', 'SM' => '378', 'SN' => '221', 'SO' => '252', 'SR' => '597', 'SS' => '211', 'ST' => '239', 'SV' => '503', 'SX' => '599', 'SY' => '963', 'SZ' => '268', 'TC' => '1649', 'TD' => '235', 'TF' => '32', 'TG' => '228', 'TH' => '66', 'TJ' => '992', 'TK' => '690', 'TL' => '670', 'TM' => '993', 'TN' => '216', 'TO' => '676', 'TR' => '90', 'TT' => '1868', 'TV' => '688', 'TW' => '886', 'TZ' => '255', 'UA' => '380', 'UG' => '256', 'UM' => '1', 'US' => '1', 'UY' => '598', 'UZ' => '998', 'VA' => '39', 'VC' => '1784', 'VE' => '58', 'VG' => '1284', 'VI' => '1340', 'VN' => '84', 'VU' => '678', 'WF' => '681', 'WS' => '685', 'YE' => '967', 'YT' => '262', 'ZA' => '27', 'ZM' => '260', 'ZW' => '263'];

		$trigger_country = get_option('fraudlabs_pro_sms_verification_wc_trigger_countries');

		switch (get_option('fraudlabs_pro_sms_verification_wc_sms_trigger')) {
			case 'new_client_only' :
				if (!$this->is_customer_ordered()) {
					$this->write_debug_log('SMS Verification on WooCommerce Checkout form triggered for new client.');
					wc_add_notice(__("You are required to verify your phone number via the SMS verification. <a href='#flp_sms_inline'>Please click here to complete the verification.</a>"), 'error');
				}
				break;

			case 'free_email' :
				$free_email = array('gmail', 'yahoo', 'hotmail', 'outlook');
				if ($_POST['billing_email']) {
					$email_domain = strtok(substr($_POST['billing_email'], strpos($_POST['billing_email'], '@') + 1), '.');
					if (in_array($email_domain, $free_email)) {
						$this->write_debug_log('SMS Verification on WooCommerce Checkout form triggered for free email.');
						wc_add_notice(__("You are required to verify your phone number via the SMS verification. <a href='#flp_sms_inline'>Please click here to complete the verification.</a>"), 'error');
					}
				}
				break;

			case 'new_device' :
				$id = $this->hash_string($_SERVER['HTTP_USER_AGENT']);
				if (!isset($_COOKIE['flp_sms_device_id'])) {
					$this->write_debug_log('SMS Verification on WooCommerce Checkout form triggered for new device.');
					wc_add_notice(__("You are required to verify your phone number via the SMS verification. <a href='#flp_sms_inline'>Please click here to complete the verification.</a>"), 'error');
				} elseif ($_COOKIE['flp_sms_device_id'] != $id) {
					$this->write_debug_log('SMS Verification on WooCommerce Checkout form triggered for new device.');
					wc_add_notice(__("You are required to verify your phone number via the SMS verification. <a href='#flp_sms_inline'>Please click here to complete the verification.</a>"), 'error');
				}
				break;

			case 'specific_billing_countries' :
				if ($_POST['billing_country']) {
					if (in_array($_POST['billing_country'], get_option('fraudlabs_pro_sms_verification_wc_trigger_countries'))) {
						$this->write_debug_log('SMS Verification on WooCommerce Checkout form triggered for specific Billing countries.');
						wc_add_notice(__("You are required to verify your phone number via the SMS verification. <a href='#flp_sms_inline'>Please click here to complete the verification.</a>"), 'error');
					}
				}
				break;

			case 'specific_shipping_countries' :
				if ($_POST['shipping_country']) {
					if (in_array($_POST['shipping_country'], get_option('fraudlabs_pro_sms_verification_wc_trigger_countries'))) {
						$this->write_debug_log('SMS Verification on WooCommerce Checkout form triggered for specific Shipping countries.');
						wc_add_notice(__("You are required to verify your phone number via the SMS verification. <a href='#flp_sms_inline'>Please click here to complete the verification.</a>"), 'error');
					}
				}
				break;

			case 'specific_idd_countries' :
				if ($_POST['billing_phone']) {
					foreach (get_option('fraudlabs_pro_sms_verification_wc_trigger_countries') as $country) {
						if (substr($_POST['billing_phone'], 0, strlen($idd[$country])) == $idd[$country]) {
							$this->write_debug_log('SMS Verification on WooCommerce Checkout form triggered for specific IDD countries.');
							wc_add_notice(__("You are required to verify your phone number via the SMS verification. <a href='#flp_sms_inline'>Please click here to complete the verification.</a>"), 'error');
							break;
						}
					}
				}
				break;

			case 'always' :
			default :
				$this->write_debug_log('Always trigger SMS Verification on WooCommerce Checkout form.');
				wc_add_notice(__("You are required to verify your phone number via the SMS verification. <a href='#flp_sms_inline'>Please click here to complete the verification.</a>"), 'error');
		}
	}

	/**
	 * Insert SMS authentication codes to WC Checkout form.
	 */
	public function woocommerce_checkout_form() {
		if (get_option('fraudlabs_pro_sms_verification_wc_default_checkout_form') == 0) {
			$this->write_debug_log('SMS Verification on WooCommerce Checkout Form not enabled. SMS Verification will not be performed.');
			return;
		}
		elseif ((trim(get_option('fraudlabs_pro_sms_verification_api_key')) == '')) {
			$this->write_debug_log('FraudLabs Pro API Key not inserted. SMS Verification will not be performed.');
			return;
		}

		$table_name = $this->create_sms_table();

		// Dynamic field name for authentication.
		$flp_ajax_field = 'flp_ajax_' . substr(hash('sha256', uniqid()), 8, 24);

		$ip = $this->get_user_ip();

		if (isset($_POST["tel_num"])) {
			$pho = sanitize_text_field($_POST["tel_num"]);

			$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);

			if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
				// Reset counter.
				$this->replace_sms_table_tel($table_name, $ip, get_option('fraudlabs_pro_sms_verification_sms_retries'), time(), $flp_ajax_field, $pho);
				$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);
			}
			else {
				// Just update ajax field.
				$this->replace_sms_table_tel($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field, $pho);
			}
			// Refresh data.
			$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);
		} else {
			$smsdata = $this->query_sms_table($table_name, $ip);

			if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
				// Reset counter.
				$this->replace_sms_table($table_name, $ip, get_option('fraudlabs_pro_sms_verification_sms_retries'), time(), $flp_ajax_field);
				$smsdata = $this->query_sms_table($table_name, $ip);
			}
			else {
				// Just update ajax field.
				$this->replace_sms_table($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field);
			}
			// Refresh data.
			$smsdata = $this->query_sms_table($table_name, $ip);
		}

		$retries = $smsdata->counter;
		$msgSmsRequired = (get_option('fraudlabs_pro_sms_verification_msg_sms_required') == '') ? 'You are required to verify you phone number via SMS verification.' : get_option('fraudlabs_pro_sms_verification_msg_sms_required');
		echo '<div id="flp-sms-verification" class="mfp-hide white-popup">';
		echo '<div id="flp_sms" style="text-align:center;"><img src="' . plugins_url('/assets/images/flp-get-otp.png', dirname(__FILE__)) . '" style="margin:15px auto; width:130px; margin-bottom:20px;"><h3 id="verifysms" style="margin-bottom:10px;">Verify Phone Number <small>(via SMS Verification)</small></h3>';
		echo '<span id="sms_status">' . (($retries < 0) ? "Maximum number of retries to send verification SMS exceeded. Please try again later.<br /><br />" : '') . '</span>';
		echo '<div id="sms_err" style="background-color:#f8d7da; color:#7d5880; padding:10px; margin-bottom:10px; font-size:1em; display:none;"></div>';
		if (($retries >= 0)) {
			if (!empty(get_option('fraudlabs_pro_sms_verification_sms_instruction'))) echo '<span id="sms_info_display">' . nl2br(get_option('fraudlabs_pro_sms_verification_sms_instruction')) . ' <br/><br/></span>';
			echo '<div><label for="phone_number" id="enter_phone_number" style="margin-top: 15px; margin-bottom: 15px;"><input type="text" name="phone_number" id="phone_number" value="" style="padding-top:8px; padding-bottom:8px; width:100%; font-size:19px;"/></label></div>';
		}
		echo '<span id="sms_phone_info" style="display:none;"></span>';
		echo '<div id="sms_otp" style="margin-top:5px; display:none;"><input type="text" name="sms_otp1" id="sms_otp1" value="" style="width:110px; background-color:white; border: none; box-shadow:none !important; font-family:Courier New, Courier, monospace !important; font-size:20px; display:inline-block;">&nbsp;&nbsp;-&nbsp;&nbsp;<input type="text" name="sms_otp2" id="sms_otp2" value="" placeholder="Enter OTP numbers" style="padding:9px 10px; font-size:17px; width:200px; display:inline-block;"></div>';
		echo '<input type="button" class="button alt" name="flp_submit_otp" id="flp_submit_otp" value="Verify" style="padding:10px 12px; width:54%; margin-top:12px; display: none;">';
		echo '<input type="button" class="button alt" name="flp_get_otp" id="flp_get_otp" value="Request OTP" style="padding:10px 12px; margin:12px auto; width:55%;' . (($retries < 0) ? ' display:none;' : '') . '">';
		echo '<div name="flp_resend_otp" id="flp_resend_otp" style="display:none; margin:10px auto;">Didn\'t receive OTP? <a href="javascript:void(0);" ><u>Resend OTP</u></a></div>';
		echo '<input type="hidden" name="sms_verified" id="sms_verified" value="">';
		echo '<input type="hidden" name="sms_tran_id" id="sms_tran_id" value="">';
		echo '<input type="hidden" name="sms_phone_cc" id="sms_phone_cc" value="' . get_option('fraudlabs_pro_sms_verification_sms_tel_cc') . '">';
		echo '<input type="hidden" name="sms_retries" id="sms_retries" value="' . $retries . '">';
		echo '<input type="hidden" name="sms_cnt" id="sms_cnt" value="1">';
		echo '</div>';
		echo '<div id="flp_sms_success_status" style="padding:20px; text-align:center; display:none;"><img src="' . plugins_url('/assets/images/flp-success-icon.png', dirname(__FILE__)) . '" style="width:160px; margin:10px auto;"><h3 style="margin-bottom:10px;">Your phone number has been successfully verified.</h3></div>';
		echo '</div>';

		echo '<div id="flp_sms_inline" style="display:none; margin:15px auto; background:#fff; box-shadow:0 5px 11px 0 rgba(0,0,0,0.18), 0 4px 15px 0 rgba(0,0,0,0.15); padding:1.1rem;"><h3 id="verifysms">SMS Verification</h3>';
		echo '<div id="flp_sms_box" style="font-size:15px; padding:5px;">';
		echo "<div id='flp_box_link'><span>" . $msgSmsRequired . "</span><br/><br/>";
		echo '<input type="button" name="verify_flp_sms_inline" id="verify_flp_sms_inline" data-mfp-src="#flp-sms-verification" class="open-popup" href="javascript:;" class="button" value="Verify Your Phone Number Now" style="white-space:normal; padding:7px 14px;' . (($retries < 0) ? ' display:none;' : '') . '"></div></div>';
		echo '<div id="flp_box_success" style="display:none; padding:5px; margin-top:10px; margin-bottom:10px;"><img src="' . plugins_url('assets/images/tick.png', dirname(__FILE__)) . '" style="display: block; margin-left: auto; margin-right: auto;"></div>';
		echo '</div><br/>';

		$msgOtpSuccess = (get_option('fraudlabs_pro_sms_verification_msg_otp_success') == '') ? 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.' : get_option('fraudlabs_pro_sms_verification_msg_otp_success');
		$msgOtpSuccess = explode("{phone}", $msgOtpSuccess);
		$msgOtpFail = (get_option('fraudlabs_pro_sms_verification_msg_otp_fail') == '') ? 'Error: Unable to send the SMS verification message to {phone}.' : get_option('fraudlabs_pro_sms_verification_msg_otp_fail');
		$msgOtpFail = explode("{phone}", $msgOtpFail);
		$msgInvalidPhone = (get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') == '') ? 'Please enter a valid phone number.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_phone');
		$msgInvalidOtp = (get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') == '') ? 'Error: Invalid OTP. Please enter the correct OTP.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_otp');
		echo '
		<script src="//ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>

		<script src="https://cdnjs.cloudflare.com/ajax/libs/magnific-popup.js/1.1.0/jquery.magnific-popup.min.js"></script>
		<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/magnific-popup.js/1.1.0/magnific-popup.min.css"/>

		<script src="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/js/intlTelInput.min.js"></script>
		<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/css/intlTelInput.min.css">

		<style>
			input[type=text]:disabled {background:#dddddd;}
			.white-popup {position: relative;background: #FFF; padding: 22px; width:auto; max-width: 630px; height:; margin: 20px auto; overflow-y:auto;z-index:99999;}
			#flp-sms{padding:23px;}
			#sms_otp2{font-size:18px;}
			#verify_flp_sms_inline{font-size:13px;}
			.iti--allow-dropdown input, .iti--allow-dropdown input[type=tel]{padding-left:94px!important;}
			@media screen and (max-width: 480px) { #flp_get_otp{font-size:14px;}#flp-sms{padding:10px;}#verify_flp_sms_inline{font-size:10px;}}
			@media (width: 1024px) {.mfp-container{top:-250px ! important;}}
			@media (width: 768px) {.mfp-container{top:-100px ! important;}}
		</style>

		<script language="Javascript">
			var phoneNum;
			jQuery(document).ready(function() {
				phoneNum = window.intlTelInput(document.querySelector("#phone_number"), {
					utilsScript: "https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/js/utils.min.js",
					separateDialCode: true,
					initialCountry: jQuery("#sms_phone_cc").val()
				});
				jQuery(".open-popup").magnificPopup({
					type:"inline",
					midClick: true
				});
			});
			jQuery(document.body).on("checkout_error", function() {
				var error_text = jQuery(".woocommerce-error").find("li").first().text();
				if (error_text.includes("You are required to verify your phone number via the SMS verification.")) {
					jQuery("#flp_sms_inline").show();
					if (jQuery.trim(jQuery("#phone_number").val()) == "") {
						var phone_num_bill = jQuery("#billing_phone").val();
						jQuery("#phone_number").val(phone_num_bill);
					}
					jQuery(\'form[name="checkout"]\').submit(function(event) {
						if (jQuery.trim(jQuery("#sms_verified").val()) == "") {
							// not yet verified
							event.preventDefault();
							event.stopImmediatePropagation();
							alert("Please complete the SMS Verification.");
							jQuery("#sms_otp").focus();
							document.getElementById("verifysms").scrollIntoView(true);
						}
					});
				}
			});
			function getPhoneNum() {
				var data = {
					"tel_num": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					data: data,
					dataType: "text"
				});
			}
			function doOTP() {
				var data = {
					"action": "wc_sms_action_send",
					"tel": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success,
					error: sms_error,
					dataType: "text"
				});
			}
			function checkOTP() {
				var data = {
					"action": "wc_sms_action_verify",
					"otp": jQuery("#sms_otp1").val() + "-" + jQuery("#sms_otp2").val(),
					"tran_id": jQuery("#sms_tran_id").val(),
					"source": "wc_checkout"
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success2,
					error: sms_error2,
					dataType: "text"
				});
			}
			jQuery("#sms_otp2").bind("keypress", function(e) {
				var code = e.keyCode || e.which;
				if (code == 13) { //Enter keycode
					e.preventDefault();
				}
			});
			jQuery("#flp_get_otp").click(function(e) {
				if (jQuery("#phone_number").val() == "") {
					jQuery("#sms_err").html("'; echo $msgInvalidPhone; echo'");
					jQuery("#sms_err").show();
					jQuery("#phone_number").focus();
				} else if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					getPhoneNum();
					doOTP();
					jQuery("#sms_cnt").val(jQuery("#sms_cnt").val());
				}
			});
			jQuery("#flp_resend_otp").click(function(e) {
				if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_submit_otp").click(function() {
				checkOTP();
			});
			function sms_success(data) {
				// need to count down the number of retries
				jQuery("#sms_retries").val(jQuery("#sms_retries").val() - 1);
				if (data.indexOf("ERROR") == 0) {
					jQuery("#sms_err").html("'; echo $msgOtpFail[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpFail[1]; echo'");
					jQuery("#sms_err").show();
					if (data.substr(10,1) == "-") {
						jQuery("#sms_otp").hide();
						jQuery("#flp_submit_otp").hide();
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
						jQuery("#sms_status").html("Maximum number of retries to send verification SMS exceeded. Please try again later.<br /><br />");
					}
				} else if (data.indexOf("OK") == 0) {
					// store the tran_id
					jQuery("#sms_tran_id").val(data.substr(2,20));
					// show the submit otp button
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").show();
					jQuery("#flp_submit_otp").show();
					jQuery("#sms_otp").show();
					jQuery("#sms_phone_info").html("'; echo $msgOtpSuccess[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpSuccess[1]; echo'<br/>");
					jQuery("#sms_phone_info").show();
					jQuery("#sms_info_display").hide();
					jQuery("#enter_phone_number").hide();
					jQuery("#sms_err").hide();
					jQuery("#sms_otp1").val(data.substr(22,6));
					jQuery("#sms_otp1").prop("disabled", true);
					if (data.substr(28,1) == "-") {
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
					}
				}
			}
			function sms_error() {
				jQuery("#sms_err").html("'; echo $msgOtpFail[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpFail[1]; echo'");
				jQuery("#sms_err").show();
			}
			function sms_success2(data) {
				if (data.indexOf("ERROR 601") == 0) {
					jQuery("#sms_err").html("'; echo $msgInvalidOtp; echo '");
					jQuery("#sms_err").show();
				} else if (data.indexOf("ERROR 600") == 0) {
					jQuery("#sms_err").html("Error: Error while performing verification.");
					jQuery("#sms_err").show();
				} else if (data.indexOf("OK") == 0) {
					// update the sms verified field
					jQuery("#sms_verified").val("YES");
					if(typeof(Storage) !== "undefined") {
						sessionStorage.wc_checkout_sms_vrf = "YES";
					}
					// hide all buttons and text field for the SMS...
					jQuery("#sms_otp").hide();
					jQuery("#flp_submit_otp").hide();
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").hide();
					jQuery("#sms_phone_info").hide();
					jQuery("#sms_err").hide();
					// show SMS verified message
					jQuery("#flp_sms_box").hide();
					jQuery("#flp_sms").hide();
					jQuery("#flp_sms_success_status").show();
					jQuery("#flp_box_link").hide();
					jQuery("#flp_box_success").show();
				}
			}
			function sms_error2() {
				jQuery("#sms_err").html("Error: Could not perform SMS verification.");
				jQuery("#sms_err").show();
			}
			if(sessionStorage.wc_checkout_sms_vrf == "YES"){
				jQuery("#sms_verified").val("YES");
				jQuery("#sms_otp").hide();
				jQuery("#flp_submit_otp").hide();
				jQuery("#flp_get_otp").hide();
				jQuery("#flp_resend_otp").hide();
				jQuery("#reset_pn").hide();
				jQuery("#sms_phone_info").hide();
				jQuery("#sms_err").hide();
				jQuery("#flp_sms_box").hide();
				jQuery("#flp_sms").hide();
				jQuery("#flp_sms_success_status").show();
				jQuery("#flp_box_link").hide();
				jQuery("#flp_box_success").show();
			}
		</script>
		';
	}

	/**
	 * Insert SMS authentication codes to WC registration form.
	 */
	public function woocommerce_register_form() {
		if ((get_option('fraudlabs_pro_sms_verification_wc_default_registration_form') == 0)) {
			$this->write_debug_log('SMS Verification on WooCommerce Registration Form not enabled. SMS Verification will not be performed.');
			return;
		}
		elseif ((get_option('fraudlabs_pro_sms_verification_api_key') == '')) {
			$this->write_debug_log('FraudLabs Pro API Key not inserted. SMS Verification will not be performed.');
			return;
		}

		$table_name = $this->create_sms_table();

		// Dynamic field name for authentication.
		$flp_ajax_field = 'flp_ajax_' . substr(hash('sha256', uniqid()), 8, 24);

		$ip = $this->get_user_ip();

		if (isset($_POST["tel_num"])) {
			$pho = sanitize_text_field($_POST["tel_num"]);

			$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);

			if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
				// Reset counter.
				$this->replace_sms_table_tel($table_name, $ip, get_option('fraudlabs_pro_sms_verification_sms_retries'), time(), $flp_ajax_field, $pho);
				$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);
			}
			else {
				// Just update ajax field.
				$this->replace_sms_table_tel($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field, $pho);
			}
			// Refresh data.
			$smsdata = $this->query_sms_table_tel($table_name, $ip, $pho);
		} else {
			$smsdata = $this->query_sms_table($table_name, $ip);

			if (($smsdata === NULL) || ($smsdata->last < strtotime('-10 minutes'))) {
				// Reset counter.
				$this->replace_sms_table($table_name, $ip, get_option('fraudlabs_pro_sms_verification_sms_retries'), time(), $flp_ajax_field);
				$smsdata = $this->query_sms_table($table_name, $ip);
			}
			else {
				// Just update ajax field.
				$this->replace_sms_table($table_name, $ip, $smsdata->counter, $smsdata->last, $flp_ajax_field);
			}
			// Refresh data.
			$smsdata = $this->query_sms_table($table_name, $ip);
		}

		$retries = $smsdata->counter;
		$msgSmsRequired = (get_option('fraudlabs_pro_sms_verification_msg_sms_required') == '') ? 'You are required to verify you phone number via SMS verification.' : get_option('fraudlabs_pro_sms_verification_msg_sms_required');
		echo '<div id="flp-sms-verification" class="mfp-hide white-popup">';
		echo '<div id="flp_sms" style="text-align:center;"><img src="' . plugins_url('/assets/images/flp-get-otp.png', dirname(__FILE__)) . '" style="margin:15px auto;width:130px;margin-bottom:20px;"><h3 id="verifysms" style="margin-bottom:10px;">Verify Phone Number <small>(via SMS Verification)</small></h3>';
		echo '<span id="sms_status">' . (($retries < 0) ? "Maximum number of retries to send verification SMS exceeded. Please try again later.<br /><br />" : '') . '</span>';
		echo '<div id="sms_err" style="background-color:#f8d7da;color:#7d5880;padding:10px;margin-bottom:10px;font-size:1em;display:none;"></div>';
		if (($retries >= 0)) {
			if (!empty(get_option('fraudlabs_pro_sms_verification_sms_instruction'))) echo '<span id="sms_info_display">' . nl2br(get_option('fraudlabs_pro_sms_verification_sms_instruction')) . ' <br/><br/></span>';
			echo '<div><label for="phone_number" id="enter_phone_number" style="margin-top: 15px; margin-bottom: 15px;"><input type="text" name="phone_number" id="phone_number" value="" style="padding-top:8px;padding-bottom:8px;width:100%;font-size:19px;"/></label></div>';
		}
		echo '<span id="sms_phone_info" style="display:none;"></span>';
		echo '<div id="sms_otp" style="margin-top:5px;display: none;"><input type="text" name="sms_otp1" id="sms_otp1" value="" style="width:110px; background-color:white; border: none; box-shadow:none !important; font-family: Courier New, Courier, monospace !important; font-size:20px;display:inline-block;">&nbsp;&nbsp;-&nbsp;&nbsp;<input type="text" name="sms_otp2" id="sms_otp2" value="" placeholder="Enter OTP numbers" style="padding:9px 10px;font-size:17px;width:200px;display:inline-block"></div>';
		echo '<input type="button" class="button alt" name="flp_submit_otp" id="flp_submit_otp" value="Verify" style="padding:10px 12px;width:54%;margin-top:12px;display: none;">';
		echo '<input type="button" class="button alt" name="flp_get_otp" id="flp_get_otp" value="Request OTP" style="padding:10px 12px;margin:12px auto;width:55%;' . (($retries < 0) ? 'display: none;' : '') . '">';
		echo '<div name="flp_resend_otp" id="flp_resend_otp" style="display:none;margin:10px auto;">Didn\'t receive OTP? <a href="javascript:void(0);" ><u>Resend OTP</u></a></div>';
		echo '<input type="hidden" name="sms_verified" id="sms_verified" value="">';
		echo '<input type="hidden" name="sms_tran_id" id="sms_tran_id" value="">';
		echo '<input type="hidden" name="sms_phone_cc" id="sms_phone_cc" value="' . get_option('fraudlabs_pro_sms_verification_sms_tel_cc') . '">';
		echo '<input type="hidden" name="sms_retries" id="sms_retries" value="' . $retries . '">';
		echo '<input type="hidden" name="sms_cnt" id="sms_cnt" value="1">';
		echo '</div>';
		echo '<div id="flp_sms_success_status" style="padding: 20px;text-align:center; display: none;"><img src="' . plugins_url('/assets/images/flp-success-icon.png', dirname(__FILE__)) . '" style="width:160px;margin:10px auto;"><h3 style="margin-bottom:10px;">Your phone number has been successfully verified.</h3></div>';
		echo '</div>';

		echo '<div id="flp_sms_inline" style="margin:15px auto;background:#fff;box-shadow: 0 5px 11px 0 rgba(0,0,0,0.18), 0 4px 15px 0 rgba(0,0,0,0.15);padding:1.1rem;"><h3 id="verifysms">SMS Verification</h3>';
		echo '<div id="flp_sms_box" style="font-size: 15px; padding: 5px;">';
		echo "<div id='flp_box_link'><span>" . $msgSmsRequired . "</span><br/><br/>";
		echo '<input type="button" name="verify_flp_sms_inline" id="verify_flp_sms_inline" data-mfp-src="#flp-sms-verification" class="open-popup" href="javascript:;" class="button" value="Verify Your Phone Number Now" style="white-space: normal;padding:7px 14px;' . (($retries < 0) ? 'display: none;' : '') . '"></div></div>';
		echo '<div id="flp_box_success" style="display:none; padding: 5px;margin-top:10px;margin-bottom:10px;"><img src="' . plugins_url('assets/images/tick.png', dirname(__FILE__)) . '" style="display: block; margin-left: auto; margin-right: auto;"></div>';
		echo '</div><br/>';

		$msgOtpSuccess = (get_option('fraudlabs_pro_sms_verification_msg_otp_success') == '') ? 'A SMS containing the OTP (One Time Passcode) has been sent to {phone}. Please enter the 6 digits OTP value to complete the verification.' : get_option('fraudlabs_pro_sms_verification_msg_otp_success');
		$msgOtpSuccess = explode("{phone}", $msgOtpSuccess);
		$msgOtpFail = (get_option('fraudlabs_pro_sms_verification_msg_otp_fail') == '') ? 'Error: Unable to send the SMS verification message to {phone}.' : get_option('fraudlabs_pro_sms_verification_msg_otp_fail');
		$msgOtpFail = explode("{phone}", $msgOtpFail);
		$msgInvalidPhone = (get_option('fraudlabs_pro_sms_verification_msg_invalid_phone') == '') ? 'Please enter a valid phone number.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_phone');
		$msgInvalidOtp = (get_option('fraudlabs_pro_sms_verification_msg_invalid_otp') == '') ? 'Error: Invalid OTP. Please enter the correct OTP.' : get_option('fraudlabs_pro_sms_verification_msg_invalid_otp');
		echo '
		<script src="//ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>

		<script src="https://cdnjs.cloudflare.com/ajax/libs/magnific-popup.js/1.1.0/jquery.magnific-popup.min.js"></script>
		<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/magnific-popup.js/1.1.0/magnific-popup.min.css"/>

		<script src="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/js/intlTelInput.min.js"></script>
		<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/css/intlTelInput.min.css">

		<style>
			input[type=text]:disabled {	background: #dddddd;}
			.white-popup {position: relative;background: #FFF;	padding: 22px;	width:auto;	max-width: 630px; height:; margin: 20px auto; overflow-y:auto;z-index:99999;}
			#flp-sms{padding:23px;}
			#verify_flp_sms_inline{font-size:13px;}
			.iti--allow-dropdown input, .iti--allow-dropdown input[type=tel]{padding-left:94px!important;}
			@media screen and (max-width: 480px) { #flp_get_otp {font-size:14px;}#flp-sms{padding:10px;}#verify_flp_sms_inline{font-size:10px;}}
			@media (width: 1024px) {.mfp-container{top:-250px ! important;}}
			@media (width: 768px) {.mfp-container{top:-100px ! important;}}
		</style>

		<script language="Javascript">
			var phoneNum;
			jQuery(document).ready(function() {
				phoneNum = window.intlTelInput(document.querySelector("#phone_number"), {
					utilsScript: "https://cdnjs.cloudflare.com/ajax/libs/intl-tel-input/17.0.12/js/utils.min.js",
					separateDialCode: true,
					initialCountry: jQuery("#sms_phone_cc").val()
				});
				jQuery(".open-popup").magnificPopup({
					type:"inline",
					midClick: true
				});
			});
			function getPhoneNum() {
				var data = {
					"tel_num": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					data: data,
					dataType: "text"
				});
			}
			function doOTP() {
				var data = {
					"action": "wc_sms_action_send",
					"tel": phoneNum.getNumber()
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success,
					error: sms_error,
					dataType: "text"
				});
			}
			function checkOTP() {
				var data = {
					"action": "wc_sms_action_verify",
					"otp": jQuery("#sms_otp1").val() + "-" + jQuery("#sms_otp2").val(),
					"tran_id": jQuery("#sms_tran_id").val(),
					"source": "wc_register"
				};
				jQuery.ajax({
					type: "POST",
					url: "' . admin_url("admin-ajax.php") . '",
					data: data,
					success: sms_success2,
					error: sms_error2,
					dataType: "text"
				});
			}
			jQuery("#sms_otp2").bind("keypress", function(e) {
				var code = e.keyCode || e.which;
				if (code == 13) { //Enter keycode
					e.preventDefault();
				}
			});
			jQuery("#flp_get_otp").click(function(e) {
				if (jQuery("#phone_number").val() == "") {
					jQuery("#sms_err").html("'; echo $msgInvalidPhone; echo'");
					jQuery("#sms_err").show();
					jQuery("#phone_number").focus();
				}else if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					getPhoneNum();
					doOTP();
					jQuery("#sms_cnt").val(jQuery("#sms_cnt").val());
				}
			});
			jQuery("#flp_resend_otp").click(function(e) {
				if (!confirm("Send OTP to " + phoneNum.getNumber() + "?")) {
					e.preventDefault();
				} else {
					doOTP();
				}
			});
			jQuery("#flp_submit_otp").click(function() {
				checkOTP();
			});
			jQuery(".register").submit(function(event) {
				if (jQuery.trim(jQuery("#sms_verified").val()) == "") {
					// not yet verified
					event.preventDefault();
					event.stopImmediatePropagation();
					alert("Please complete the SMS Verification!");
					jQuery("#sms_otp1").focus();
					document.getElementById("verifysms").scrollIntoView(true);
				}
			});
			function sms_success(data) {
				// need to count down the number of retries
				jQuery("#sms_retries").val(jQuery("#sms_retries").val() - 1);
				if (data.indexOf("ERROR") == 0) {
					jQuery("#sms_err").html("'; echo $msgOtpFail[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpFail[1]; echo'");
					jQuery("#sms_err").show();
					if (data.substr(10,1) == "-") {
						jQuery("#sms_otp").hide();
						jQuery("#flp_submit_otp").hide();
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
						jQuery("#sms_status").html("Maximum number of retries to send verification SMS exceeded. Please try again later.<br /><br />");
					}
				} else if (data.indexOf("OK") == 0) {
					// store the tran_id
					jQuery("#sms_tran_id").val(data.substr(2,20));
					// show the submit otp button
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_submit_otp").show();
					jQuery("#flp_resend_otp").show();
					//jQuery("#sms_otp").attr(\'style\', \'display: inline-block !important\');
					jQuery("#sms_otp").show();
					jQuery("#sms_phone_info").html("'; echo $msgOtpSuccess[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpSuccess[1]; echo'<br/>");
					jQuery("#sms_phone_info").show();
					jQuery("#sms_info_display").hide();
					jQuery("#enter_phone_number").hide();
					jQuery("#sms_err").hide();
					jQuery("#sms_otp1").val(data.substr(22,6));
					jQuery("#sms_otp1").prop("disabled", true);
					if (data.substr(28,1) == "-") {
						jQuery("#flp_get_otp").hide();
						jQuery("#flp_resend_otp").hide();
					}
				}
			}
			function sms_error() {
				jQuery("#sms_err").html("'; echo $msgOtpFail[0]; echo'" + phoneNum.getNumber() + "'; echo $msgOtpFail[1]; echo'");
				jQuery("#sms_err").show();
			}
			function sms_success2(data) {
				if (data.indexOf("ERROR 601") == 0) {
					jQuery("#sms_err").html("'; echo $msgInvalidOtp; echo '");
					jQuery("#sms_err").show();
				} else if (data.indexOf("ERROR 600") == 0) {
					jQuery("#sms_err").html("Error: Error while performing verification.");
					jQuery("#sms_err").show();
				} else if (data.indexOf("OK") == 0) {
					// update the sms verified field
					jQuery("#sms_verified").val("YES");
					if(typeof(Storage) !== "undefined") {
						sessionStorage.wc_register_sms_vrf = "YES";
					}
					// hide all buttons and text field for the SMS...
					jQuery("#sms_otp").hide();
					jQuery("#flp_submit_otp").hide();
					jQuery("#flp_get_otp").hide();
					jQuery("#flp_resend_otp").hide();
					jQuery("#sms_phone_info").hide();
					jQuery("#sms_err").hide();
					// show SMS verified message
					jQuery("#flp_sms_box").hide();
					jQuery("#flp_sms").hide();
					jQuery("#flp_sms_success_status").show();
					jQuery("#flp_box_link").hide();
					jQuery("#flp_box_success").show();
				}
			}
			function sms_error2() {
				jQuery("#sms_err").html("Error: Could not perform SMS verification.");
				jQuery("#sms_err").show();
			}
			if(sessionStorage.wc_register_sms_vrf == "YES"){
				jQuery("#sms_verified").val("YES");
				jQuery("#sms_otp").hide();
				jQuery("#flp_submit_otp").hide();
				jQuery("#flp_get_otp").hide();
				jQuery("#flp_resend_otp").hide();
				jQuery("#reset_pn").hide();
				jQuery("#sms_phone_info").hide();
				jQuery("#sms_err").hide();
				jQuery("#flp_sms_box").hide();
				jQuery("#flp_sms").hide();
				jQuery("#flp_sms_success_status").show();
				jQuery("#flp_box_link").hide();
				jQuery("#flp_box_success").show();
			}
		</script>
		';
	}

	/**
	 * Enqueue the script in dashboard.
	 */
	public function plugin_enqueues() {
		if (is_admin() && get_user_meta(get_current_user_id(), 'fraudlabspro_sms_verification_wc_admin_notice', true) !== 'dismissed') {

			wp_enqueue_script('fraudlabspro_sms_verification_wc_admin_script', plugins_url('/assets/js/notice-update.js', dirname(__FILE__)), array('jquery'), '1.0', true);

			wp_localize_script('fraudlabspro_sms_verification_wc_admin_script', 'fraudlabspro_sms_verification_wc_admin', array('fraudlabspro_sms_verification_wc_admin_nonce' => wp_create_nonce('fraudlabspro_sms_verification_wc_admin_nonce'),));
		}
	}

	/**
	 * Add notification in dashboard.
	 */
	public function admin_notifications() {
		if (get_user_meta(get_current_user_id(), 'fraudlabspro_sms_verification_wc_admin_notice', true) === 'dismissed') {
			return;
		}

		$currentscr = get_current_screen();

		if ('plugins' == $currentscr->parent_base) {
			$settings_url = admin_url('admin.php?page=fraudlabspro-sms-verification&tab=woocommerce');

			echo '
			<div id="fraudlabspro-sms-verification-wc-notice" class="notice is-dismissible">
				<p>
					' . __('FraudLabs Pro SMS Verification WooCommerce Settings page has been moved to <a href="' . $settings_url . '">FraudLabs Pro setting page</a>. Please configure your settings at the WooCommerce tab and click on the <strong>Save Changes</strong> button.', 'woocommerce-fraudlabs-pro') . '
				</p>
			</div>
			';
		}
	}

	/**
	 * Dismiss the admin notice.
	 */
	public function plugin_dismiss_admin_notice() {
		if (!isset($_POST['fraudlabspro_sms_verification_wc_admin_nonce']) || !wp_verify_nonce($_POST['fraudlabspro_sms_verification_wc_admin_nonce'], 'fraudlabspro_sms_verification_wc_admin_nonce')) {
			wp_die();
		}

		update_user_meta(get_current_user_id(), 'fraudlabspro_sms_verification_wc_admin_notice', 'dismissed');
	}

	/**
	 * Write to debug log to record details of process.
	 */
	public function write_debug_log($message) {
		if (get_option('fraudlabs_pro_sms_verification_debug_log_enabled') == 0) {
			return;
		}

		if (is_array($message) || is_object($message)) {
			file_put_contents(FRAUDLABS_PRO_SMS_VERIFICATION_ROOT . 'debug.log', gmdate('Y-m-d H:i:s') . "\t" . print_r($message, true) . "\n", FILE_APPEND);
		} else {
			file_put_contents(FRAUDLABS_PRO_SMS_VERIFICATION_ROOT . 'debug.log', gmdate('Y-m-d H:i:s') . "\t" . $message . "\n", FILE_APPEND);
		}
	}

	/**
	 * AJAX function for SMS sending.
	 */
	function wc_sms_action_send() {
		global $wpdb;
		$table_name = $wpdb->prefix . 'flp_sms_counter';
		$ip = $this->get_user_ip();
		$smsdata = $this->query_sms_table_tel($table_name, $ip, sanitize_text_field($_POST['tel']));
		if (($smsdata === NULL)) {
			$smsdata = $this->query_sms_table($table_name, $ip);
		}

		$sms_order_id = (isset($_POST['sms_order_id'])) ? sanitize_text_field($_POST['sms_order_id']) : "";
		if ($sms_order_id != "") {
			$result = get_post_meta($sms_order_id, '_fraudlabspro');
			if (!is_array($result[0]) && !is_object($result[0])) {
				$row = json_decode($result[0]);
			} else {
				$row = $result[0];
			}

			if (is_array($result[0])) {
				$flpId = $row['fraudlabspro_id'];
			} else {
				$flpId = $row->fraudlabspro_id;
			}
		} else {
			$flpId = '';
		}

		$params['source'] = 'woocommerce';
		$params['tel'] = trim(sanitize_text_field($_POST['tel']));
		if (strpos($params['tel'], '+') !== 0)
			$params['tel'] = '+' . $params['tel'];
		$params['mesg'] = get_option('fraudlabs_pro_sms_verification_sms_template');
		$params['mesg'] = str_replace(['{', '}'], ['<', '>'], $params['mesg']);
		$params['flp_id'] = $flpId;
		$params['tel_cc'] = (isset($_POST['tel_cc'])) ? sanitize_text_field($_POST['tel_cc']) : "";
		$params['otp_timeout'] = (get_option('fraudlabs_pro_sms_verification_otp_timeout') == '') ? '3600' : get_option('fraudlabs_pro_sms_verification_otp_timeout');
		$url = 'https://api.fraudlabspro.com/v1/verification/send';

		$smsdata->counter = $smsdata->counter-1;
		$this->replace_sms_table_tel($table_name, $ip, $smsdata->counter, $smsdata->last, $smsdata->ajax, sanitize_text_field($_POST['tel']));

		$query = '';
		foreach( $params as $key=>$value ) {
			$query .= '&' . $key . '=' . rawurlencode( $value );
		}
		$url = $url . '?key=' . get_option('fraudlabs_pro_sms_verification_api_key') . $query;
		$request = wp_remote_get( $url );

		// Network error, wait 2 seconds for next retry
		if (is_wp_error($request)) {
			$this->write_debug_log( 'Network error during FraudLabs Pro SMS OTP Send. Retrying.' );
			for ($i = 0; $i < 3; ++$i ) {
				sleep(2);
				$request = wp_remote_get( $url );
				
				if (!is_wp_error($request)) {
					break;
				}
			}
		}

		// Still having network issue after 3 retries
		if (is_wp_error($request)) {
			$this->write_debug_log('Network error during FraudLabs Pro SMS OTP Send. SMS Verification aborted.');
			die ('ERROR 500');
		}

		// Get the HTTP response
		$data = json_decode(wp_remote_retrieve_body($request));

		if (trim($data->error) != '') {
			$this->write_debug_log('Error occurred during FraudLabs Pro SMS OTP Send to ' . sanitize_text_field($_POST['tel']) . '. ERROR: ' . $data->error);
			die('ERROR 600 ' . $smsdata->counter);
		} else {
			die ('OK' . $data->tran_id . $data->otp_char. $smsdata->counter);
		}
	}

	/**
	 * AJAX function for SMS verifying.
	 */
	function wc_sms_action_verify() {
		global $wpdb;
		$table_name = $wpdb->prefix . 'flp_sms_counter';
		$ip = $this->get_user_ip();

		$params['otp'] = (isset($_POST['otp'])) ? sanitize_text_field($_POST['otp']) : die('ERROR 400');
		$params['tran_id'] = (isset($_POST['tran_id'])) ? sanitize_text_field($_POST['tran_id']) : die('ERROR 400');
		$src = (isset($_POST['source'])) ? sanitize_text_field($_POST['source']) : '';

		require_once FRAUDLABS_PRO_SMS_VERIFICATION_ROOT . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

		// Configures FraudLabs Pro API key
		$config = new FraudLabsPro\Configuration(get_option('fraudlabs_pro_sms_verification_api_key'));
		$fraudlabsproSms = new \FraudLabsPro\SmsVerification($config);

		$data = $fraudlabsproSms->verifyOtp([
			'tran_id'		=> $params['tran_id'],
			'otp'			=> $params['otp'],
		]);

		if (trim($data->error) != '') {
			if ($data->error == 'Invalid OTP.') {
				die('ERROR 601-' . $data->error);
			} else {
				$this->write_debug_log('Error occurred during FraudLabs Pro SMS OTP Verify. ERROR: ' . $data->error);
				die('ERROR 600-' . $data->error);
			}
		}
		else {
			$this->delete_sms_table($table_name, $ip);

			if (($src = 'wc_checkout') && ((get_option('fraudlabs_pro_sms_verification_wc_sms_trigger') == 'new_device'))) {
				$id = $this->hash_string($_SERVER['HTTP_USER_AGENT']);
				if (!isset($_COOKIE['flp_sms_device_id'])) {
					setcookie('flp_sms_device_id', $id, time() + (86400 * 90), "/");
				}
			} elseif ($src = 'wc_shortcode') {
				$sms_order_id = (isset($_POST['sms_order_id'])) ? sanitize_text_field($_POST['sms_order_id']) : "";
				$sms_code = (isset($_POST['sms_code'])) ? sanitize_text_field($_POST['sms_code']) : "";
				if ($sms_order_id != "") {
					if ($sms_code != "") {
						$result = get_post_meta($sms_order_id, '_fraudlabspro_sms_wc');
						if (!is_array($result[0]) && !is_object($result[0])) {
							$sms_verify = json_decode($result[0]);
						} else {
							$sms_verify = $result[0];
						}

						if (is_array($result[0])) {
							if ($sms_verify['fraudlabspro_wc_email_code'] == $sms_code) {
								$sms_verify['fraudlabspro_wc_email_code'] = $sms_code . '_VERIFIED';
								update_post_meta($sms_order_id, '_fraudlabspro_sms_wc', $sms_verify);
							}
						} else {
							if ($sms_verify->fraudlabspro_wc_email_code == $sms_code) {
								$sms_verify->fraudlabspro_wc_email_code = $sms_code . '_VERIFIED';
								update_post_meta($sms_order_id, '_fraudlabspro_sms_wc', $sms_verify);
							}
						}

					}
				}
			}

			die ('OK');
		}
	}

	/**
	 * Check customer for first time client.
	 */
	private function is_customer_ordered() {
		// get all customer orders
		$customer_orders = get_posts(array(
			'numberposts'	=> -1,
			'meta_key'		=> '_customer_user',
			'meta_value'	=> get_current_user_id(),
			'post_type'		=> 'shop_order',
			'post_status'	=> 'wc-completed'
		));

		$count = count($customer_orders);

		if ($count > 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Generate random code for email verification.
	 */
	private function randomCode($length=16) {
		$key = '';
		$pattern = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		for($i=0; $i<$length; $i++) {
			$key .= $pattern[rand(0, strlen($pattern)-1)];
		}
		return $key;
	}

	/**
	 * Delete data in sms table.
	 */
	private function delete_sms_table($table_name, $ip) {
		global $wpdb;

		$wpdb->delete($table_name, array('ip' => $ip), array('%s'));
	}

	/**
	 * Get user IP.
	 */
	private function get_user_ip() {
		if (isset($_SERVER['HTTP_X_FORWARDED_FOR']) && $this->isvalidip($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			$ip_address = $_SERVER['HTTP_X_FORWARDED_FOR'];
		}
		else {
			$ip_address = $_SERVER['REMOTE_ADDR'];
		}

		if (preg_match('/,/', $ip_address)) {
			$tmp = explode(',', $ip_address);
			$ip_address = trim($tmp[0]);
		}
		return $ip_address;
	}

	/**
	 * Check for valid IP.
	 */
	private function isvalidip($ip) {
		if ((filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) || (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6))) {
			return true;
		}
		else {
			return false;
		}
	}

	/**
	 * Create SMS table.
	 */
	private function create_sms_table() {
		global $wpdb;
		$charset_collate = $wpdb->get_charset_collate();
		$table_name = $wpdb->prefix . 'flp_sms_counter';

		$sql = "CREATE TABLE IF NOT EXISTS $table_name (
			`ip` varchar(50) NOT NULL,
			`counter` int DEFAULT '0',
			`last` int DEFAULT '0',
			`ajax` varchar(50) DEFAULT '',
			`phone` varchar(20) DEFAULT '0',
			PRIMARY KEY (`ip`, `phone`)
		) $charset_collate;";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql); // This will check if table structure needs to be updated and do it automatically.

		return $table_name;
	}

	/**
	 * Query SMS table.
	 */
	private function query_sms_table($table_name, $ip) {
		global $wpdb;

		return $wpdb->get_row("SELECT * FROM $table_name WHERE ip = '" . $ip . "' AND phone = '0' LIMIT 1");
	}
	private function query_sms_table_tel($table_name, $ip, $phone) {
		global $wpdb;

		return $wpdb->get_row("SELECT * FROM $table_name WHERE ip = '" . $ip . "' AND phone = '" . $phone . "' LIMIT 1");
	}

	/**
	 * Update SMS table.
	 */
	private function replace_sms_table($table_name, $ip, $counter, $last, $ajax) {
		global $wpdb;

		$wpdb->replace($table_name, array('ip' => $ip, 'counter' => $counter, 'last' => $last, 'ajax' => $ajax), array('%s', '%d', '%d', '%s'));
	}
	private function replace_sms_table_tel($table_name, $ip, $counter, $last, $ajax, $phone) {
		global $wpdb;

		$wpdb->replace($table_name, array('ip' => $ip, 'counter' => $counter, 'last' => $last, 'ajax' => $ajax, 'phone' => $phone), array('%s', '%d', '%d', '%s', '%s'));
	}

	/**
	 * Hash a string.
	 */
	private function hash_string( $s ) {
		$hash = 'fraudlabspro_' . $s;

		for( $i = 0; $i < 65536; $i++ )
			$hash = sha1( 'fraudlabspro_' . $hash );

		return $hash;
	}
}

endif;
